//
///////////////////////////////////////////////////////////////////////////////
//
//    File Name:  dll_config_r1.0.v
//      Version:  1.0
//         Date:  22.02.2008
//        Model:  
//
//      Company:  Xilinx, Inc.
//  Contributor:  Martin Kellermann
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2007 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
// 
// Summary:
//  This module serves as a wrapper for the adjustable oscillator and provides
//  some hooks for debugging. In the normal use those signals are not used but
//  may be monitored using e.g. ChipScope.
//--------------------------------------------------------------------------------
`timescale 1ns / 1ps
module dll_config #(
	parameter CLOCK_BUFFER = "BUFR",
	parameter [7:0] TARGET_FREQUENCY = 125)
	(
	input CLK_200M,
	input FASTER,
	input SLOWER,
	input STOP_UPDATE,
	output DLL_CLK,
	input RESET,
	input RESET_DLY_CTRL,
	output DLY_CTRL_RDY,
	input CTRL_CLK
	);

	
	`include "./iodll_src/pkg_dll_reco_r1.0.v"
	
	wire  err_too_fast;
	wire  err_too_slow;

	
	parameter taps = start_tap_number_func(TARGET_FREQUENCY);
	parameter dll_divide = dll_divide_factor_func(TARGET_FREQUENCY);
  
	dll_osc #(
    .CLOCK_BUFFER(CLOCK_BUFFER),
    .TAP_AMOUNT(taps),
    .DLL_DIV(dll_divide))
  osc_core(
    .DLL_CLK(DLL_CLK),
    .RESET(RESET),
    .RESET_DLY_CTRL(RESET_DLY_CTRL),
    .STOP_UPDATE(STOP_UPDATE),
    .DLY_CTRL_RDY(DLY_CTRL_RDY),
    .CLK_200M(CLK_200M),
    .CTRL_CLK(CTRL_CLK),
    .ERR_TOO_FAST(err_too_fast),
    .ERR_TOO_SLOW(err_too_slow),
    .FASTER(FASTER),
    .SLOWER(SLOWER));


endmodule
