
///////////////////////////////////////////////////////////////////////////////
//
//    File Name:  dll_osc_r1.0.v
//      Version:  1.0
//         Date:  22.02.2008
//        Model:  Martin Kellermann
//
//      Company:  Xilinx, Inc.
//  Contributor:  
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2008 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
// 
// Summary:
//	This module is the core of the DLL built out of the IODELAY. The IODELAY is
//	looped back and inverted onto itself, generating an oscillation.
//	The period of the oscillaton is 2 x (number_of_taps * 80 ps)
//
//	Example (ignoring routing-delay, see package-file for detailled values): 
//	* 100 MHz: number_of_taps = 63
//	* 200 MHz: number_of_taps = 31
//	* 250 MHz: number_of_taps = 25
//	* 125 MHz: number_of_taps = 25, div2 
//
//
//--------------------------------------------------------------------------------
`timescale 1ps / 1ps

module dll_osc #(
	parameter CLOCK_BUFFER = "BUFR",
	parameter [5:0] TAP_AMOUNT = 31,
	parameter [1:0] DLL_DIV = 0)
	(
	output DLL_CLK,
	input RESET,
	input RESET_DLY_CTRL,
	input STOP_UPDATE,
	output DLY_CTRL_RDY,
	input CLK_200M,
	input CTRL_CLK,
	output ERR_TOO_FAST,
	output ERR_TOO_SLOW,
	input FASTER,
	input SLOWER
	);
	`include "./iodll_src/pkg_dll_reco_r1.0.v"

	wire  clk_int;
	reg  clk_int_not;// = 0;
	reg  clk_int_not_dly;
	
	wire  dll_clk_int;
	reg  inc;// = 0;
	reg  adj_ce;// = 0;
	reg  inc_ctrl;// = 0;
	reg  adj_ce_ctrl;// = 0;
	reg err_too_slow_int;// = 0;
	reg err_too_fast_int;// = 0;


	reg [2:0] div_cnt;// = 0;
	reg [5:0] tap_cnt;// = TAP_AMOUNT;

	parameter GND = 1'b 0;
	parameter VCC = 1'b 1;
	
	reg startup;
	
	wire [1:0] adj;   
	
	assign ERR_TOO_FAST = err_too_fast_int;
	assign ERR_TOO_SLOW = err_too_slow_int;

initial begin
	clk_int_not = 0;
	clk_int_not_dly = 0;

	inc = 0;
	adj_ce = 0;
	inc_ctrl = 0;
	adj_ce_ctrl = 0;
	err_too_slow_int = 0;
	err_too_fast_int = 0;
	div_cnt = 0;
	tap_cnt = TAP_AMOUNT;
end
//	attribute keep: string;
//	attribute keep of tap_cnt: signal is "true";
//	attribute keep of dll_clk_int: signal is "true";
//	attribute keep of DLL_CLK: signal is "true";
//	attribute buffer_type: string;
//	attribute buffer_type of dll_clk_int: signal is "NONE";

  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //
  //Local IDELAYCTRL, can be disabled in package
  //
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  generate if (LOCAL_IDELAYCTRL == True) begin 
      IDELAYCTRL dlyctrl(
          .RDY(DLY_CTRL_RDY),
          .REFCLK(CLK_200M),
          .RST(RESET_DLY_CTRL));

  end
  endgenerate
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //
  // The beating heart of the system, the oscillator
  //
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

  IODELAY #(
      .DELAY_SRC("DATAIN"),
			.HIGH_PERFORMANCE_MODE("TRUE"),
			.IDELAY_TYPE("VARIABLE"),
			.IDELAY_VALUE(TAP_AMOUNT),
			.REFCLK_FREQUENCY(200.0),
			.SIGNAL_PATTERN("CLOCK")
			)
  osc(
    .DATAOUT(clk_int),
    .C(CTRL_CLK),
    .CE(adj_ce_ctrl),
    .DATAIN(clk_int_not_dly),
    .IDATAIN(GND),
    .INC(inc_ctrl),
    .ODATAIN(GND),
    .RST(RESET),
    .T(GND));

  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //
  //Divider-stages for the output of the DLL-oscillator
  //
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  always @(posedge clk_int) begin
    div_cnt <= div_cnt + 1'b 1;
  end

  generate if (DLL_DIV == 0) begin
      assign dll_clk_int = clk_int;
  end
  endgenerate
  generate if (DLL_DIV == 1) begin
      assign dll_clk_int = div_cnt[0] ;
  end
  endgenerate
  generate if (DLL_DIV == 2) begin
      assign dll_clk_int = div_cnt[1] ;
  end
  endgenerate
  generate if (DLL_DIV == 3) begin
      assign dll_clk_int = div_cnt[2] ;
  end
  endgenerate
  generate if (CLOCK_BUFFER == "BUFG") begin
      BUFG DLL_BUFG(
          .I(dll_clk_int),
      .O(DLL_CLK));

  end
  endgenerate
  generate if (CLOCK_BUFFER == "BUFR") begin
      BUFR DLL_BUFR(
          .I(dll_clk_int),
      .CLR(GND),
      .CE(VCC),
      .O(DLL_CLK));

  end
  endgenerate
  generate if (CLOCK_BUFFER == "BUF") begin
      BUF DLL_BUF(
          .I(dll_clk_int),
      .O(DLL_CLK));

  end
  endgenerate
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //
  // Cleaning of the delay-pipe for simulation, not neccessary 
  // in hardware
  //
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	//synthesis translate_off
	always begin
		if($realtime < 2 * TAP_AMOUNT * 80)begin
			startup <= 1'b1;
			#10;
		end
		else begin
			startup <= 1'b0;
			#10;
		end 
	end
	//synthesis translate_on	
		
 
	always @(startup or clk_int) begin
		if (simulation_func(1'b 0)==True) begin
  //This process is only used to start up the oscillation in simulation
  //by avoiding having a permanently inverted 'X' in the IODELAY.
  //During implementation the 'Latch' is stripped out and does not appear.
			//synthesis translate_off			
			if ($realtime < 2 * TAP_AMOUNT * 80)begin
				clk_int_not <= 1'b 0;		
			end
			else begin
				clk_int_not <= ~clk_int;
			end
			//synthesis translate_on
		end
		else begin // 
			clk_int_not <= ~clk_int;		
		end

	end
	
  // simulated delay of the FPGA-routing which has a big effect on the freqeuncy: 
  always @ (clk_int_not) 
	begin
		clk_int_not_dly <= #1000 clk_int_not;
	end

	
	
	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //Translate the FASTER-/SLOWER command into signals for
  //the IODELAY
	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  assign adj[1]=FASTER ;
	assign adj[0]=SLOWER ;
	
	
	always @(posedge CTRL_CLK) begin : P1
    if(RESET == 1'b 1) begin
      inc <= 1'b 0;
      adj_ce <= 1'b 0;
    end
    else if(STOP_UPDATE == 1'b 0) begin
      inc <= 1'b 0;
      adj_ce <= 1'b 0;
			case(adj)
				2'b01:begin
					inc <= 1'b 1;
					adj_ce <= 1'b 1;
				end
				2'b10:begin
					inc <= 1'b 0;
					adj_ce <= 1'b 1;
				end
				default:begin
					inc <= 1'b 0;
					adj_ce <= 1'b 0;
				end
			endcase
    end
  end

  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //
  //Filter the IODELAY-commands to avoid getting too slow or fast:
  //
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  always @(posedge CTRL_CLK) begin
    if(RESET == 1'b 1) begin
			err_too_fast_int <= 1'b 0;
			err_too_slow_int <= 1'b 0;
    end
    else if(adj_ce == 1'b 1) begin
      err_too_fast_int <= 1'b 0;
			err_too_slow_int <= 1'b 0;
      if(inc == 1'b 1) begin
        //incrementing the number of taps => f(dll)getting SLOWER
        if(tap_cnt < MAX_TAP_CNT) begin
          inc_ctrl <= inc;
          adj_ce_ctrl <= 1'b 1;
        end
        else begin
					err_too_slow_int <= 1'b 1;
          $display ("DLL IODELAY getting too slow!");
        end
      end
      else begin
        //decrementing the number of taps => f(dll)getting FASTER
        inc_ctrl <= 1'b 0;
        if(tap_cnt >= MIN_TAP_CNT) begin
          adj_ce_ctrl <= 1'b 1;
        end
        else begin
          adj_ce_ctrl <= 1'b 0;
          err_too_fast_int <= 1'b 1;
          $display ("DLL IODELAY getting too fast!");
        end
      end
    end
    else if(adj_ce == 1'b 0) begin
      adj_ce_ctrl <= 1'b 0;
    end
  end

  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //
  // Counter in parallel to IODELAY to monitor the number of taps
  //
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  always @(posedge CTRL_CLK) begin
    if(RESET == 1'b 1) begin
      tap_cnt <= TAP_AMOUNT;
    end
    else if(adj_ce_ctrl == 1'b 1) begin
      if(inc_ctrl == 1'b 1) begin
        tap_cnt <= tap_cnt + 1;
      end
      else begin
        tap_cnt <= tap_cnt - 1;
      end
    end
  end


endmodule
