// 
// /////////////////////////////////////////////////////////////////////////////
// 
//     File Name:  io_dll_r1.0.v
//       Version:  1.0
//          Date:  22.02.2008
//         Model:  
// 
//       Company:  Xilinx, Inc.
//   Contributor:  Martin Kellermann
// 
//    Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                 INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                 PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                 PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                 ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                 APPLICATION OR STANDARD, XILINX IS MAKING NO
//                 REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                 FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                 RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                 REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                 EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                 RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                 INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                 REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                 FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                 PURPOSE.
// 
//                 (c) Copyright 2008 Xilinx, Inc.
//                 All rights reserved.
// 
// /////////////////////////////////////////////////////////////////////////////
//  
//  Summary:
//   This module combines the adjustable oscillator with the control-circuitry
//   to form the DLL itself.
// --------------------------------------------------------------------------------
`timescale 1ns / 1ps
module io_dll #(
	parameter CLOCK_BUFFER = "BUFR",//allowed inputs are "BUFG", "BUFR", "BUF"
	parameter [7:0] DLL_MULTIPLY = 2,
	parameter [7:0] TARGET_FREQUENCY = 125
	)
	(
	input  REFCLK,
	output DLLCLK,
	output LOCKED,
	input  RESET);


wire    dll_faster; 
wire    dll_slower; 
wire    dll_clk_int;

	 clk_adjust_v2 #(
			.DLL_MULTIPLY(DLL_MULTIPLY))
	 clk_adjuster(
          .DLL_CLK(dll_clk_int),
          .REF_CLK(REFCLK),
          .CTRL_CLK(dll_clk_int),
          .STOP_LOCK_CHECK(1'b 0),
          .DLL_FASTER(dll_faster),
          .DLL_SLOWER(dll_slower),
          .DLL_LOCKED(LOCKED),
          .RESET(RESET));
		
													
 dll_config #(
   .CLOCK_BUFFER(CLOCK_BUFFER),
	 .TARGET_FREQUENCY(TARGET_FREQUENCY))
 configurable_dll(.CLK_200M(1'b 0),
          .FASTER(dll_faster),
          .SLOWER(dll_slower),
          .STOP_UPDATE(1'b 0),
          .DLL_CLK(dll_clk_int),
          .RESET(RESET),
          .RESET_DLY_CTRL(1'b 0),
          .DLY_CTRL_RDY(),
          .CTRL_CLK(dll_clk_int));
					
assign DLLCLK = dll_clk_int; 

endmodule // module io_dll

