//
// /////////////////////////////////////////////////////////////////////////////
// 
//     File Name:  lock_count_r1.0.v
//       Version:  1.0
//          Date:  22.02.2008
//         Model:  
// 
//       Company:  Xilinx, Inc.
//   Contributor:  Martin Kellermann
// 
//    Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                 INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                 PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                 PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                 ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                 APPLICATION OR STANDARD, XILINX IS MAKING NO
//                 REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                 FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                 RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                 REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                 EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                 RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                 INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                 REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                 FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                 PURPOSE.
// 
//                 (c) Copyright 2008 Xilinx, Inc.
//                 All rights reserved.
// 
// /////////////////////////////////////////////////////////////////////////////
//  
//  Summary:
//   This module is run once the alignment-circuitry is switched into the
//   exact loop. It then counts for a defined number of  faster- and slower-
//   signals without FULL- or EMPTY-condition occuring.
// 
// --------------------------------------------------------------------------------
`timescale 1ns / 1ps
module lock_count #(
	parameter [5:0] COUNT_LOCK = 16)

(
   input FASTER,
   input SLOWER,
   input FULL,
   input EMPTY,
   input CTRL_CLK,
   input CHECK_LOCK,
   output LOCK);


reg     [3:0] adjust_cnt; 
reg lock_int;

initial begin
   adjust_cnt = 0;
	 lock_int = 0;
end


// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//
//  The lock-counter
//
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

always @(posedge CTRL_CLK)
   begin : process_1
   if (CTRL_CLK === 1'b 1)
      begin
      if (CHECK_LOCK === 1'b 1)
         begin
         lock_int <= 1'b 0;   
         if (FULL === 1'b 1 | EMPTY === 1'b 1)
            begin
            adjust_cnt <= 0;   
            end
         else if (FASTER === 1'b 1 | SLOWER === 1'b 1 )
            begin
            if (adjust_cnt !== COUNT_LOCK - 1)
               begin
               adjust_cnt <= adjust_cnt + 1;   
               end
            end
         if (adjust_cnt === COUNT_LOCK - 1)
            begin
            lock_int <= 1'b 1;   
            end
         else
            begin
            lock_int <= 1'b 0;   
            end
         end
      else
         begin
         adjust_cnt <= 0;   
         lock_int <= 1'b 0;   
         end
      end
   end

	assign LOCK = lock_int;

endmodule // module lock_count

