//
///////////////////////////////////////////////////////////////////////////////
//
//    File Name:  pkg_dll_reco_r1.0.v
//      Version:  1.0
//         Date:  22.02.2008
//        Model:
//
//      Company:  Xilinx, Inc.
//  Contributor:  Martin Kellermann
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2008 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
//
// Summary:


//User modifiable constants:

	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	//
	// Usage of a IDELAYCTRL-block instantiated inside the code of
	// the oscillator if set to "true". This is advisable if other 
	// parts of the whole design do also instantiate IDELAYCTRL-
	// blocks.
	//
	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	parameter LOCAL_IDELAYCTRL = 1'b 0;


	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	//
	//Design constants, do not change:
	//
	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	parameter MAX_TAP_CNT =63;
	parameter MIN_TAP_CNT =8;


	parameter True = 1'b 1;
	parameter False = 1'b 0;

//module pkg_dll_reco();

	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	//
	// Decision on how many taps to be used at startup
	// Number of taps =
	//   integer((Period(Mean(upper_limit, lower_limit))[ns] - 1 ns) / 0.16 ns)
	//
	// example, range 200MHz .. 224MHz:
	// * mean = 212 MHz => Period = 4.72 ns
	// * taps = (4.72 - 1) / 0.16 = 23.25 
	// ==> tap_amount = 23
	//
	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	function [5:0] start_tap_number_func;
		input [7:0] TARGET_FREQUENCY; //range 12 to 250
		integer tap_amount;
	begin
	tap_amount=0;	
		if ((TARGET_FREQUENCY >= 12) && (TARGET_FREQUENCY <= 13))
			tap_amount = 61;
		else if ((TARGET_FREQUENCY >= 14) && (TARGET_FREQUENCY <= 15))
			tap_amount = 52;
		else if ((TARGET_FREQUENCY >= 16) && (TARGET_FREQUENCY <= 18))
			tap_amount = 44;
		else if ((TARGET_FREQUENCY >= 19) && (TARGET_FREQUENCY <= 21))
			tap_amount = 38;
		else if ((TARGET_FREQUENCY >= 22) && (TARGET_FREQUENCY <= 26))
			tap_amount = 31;
		else if ((TARGET_FREQUENCY >= 27) && (TARGET_FREQUENCY <= 30))
			tap_amount = 52;
		else if ((TARGET_FREQUENCY >= 31) && (TARGET_FREQUENCY <= 34))
			tap_amount = 45;
		else if ((TARGET_FREQUENCY >= 35) && (TARGET_FREQUENCY <= 38))
			tap_amount = 40;
		else if ((TARGET_FREQUENCY >= 39) && (TARGET_FREQUENCY <= 44))
			tap_amount = 35;
		else if ((TARGET_FREQUENCY >= 45) && (TARGET_FREQUENCY <= 53))
			tap_amount = 29;
		else if ((TARGET_FREQUENCY >= 54) && (TARGET_FREQUENCY <= 49))
			tap_amount = 49;
		else if ((TARGET_FREQUENCY >= 60) && (TARGET_FREQUENCY <= 65))
			tap_amount = 44;
		else if ((TARGET_FREQUENCY >= 66) && (TARGET_FREQUENCY <= 74))
			tap_amount = 38;
		else if ((TARGET_FREQUENCY >= 75) && (TARGET_FREQUENCY <= 84))
			tap_amount = 33;
		else if ((TARGET_FREQUENCY >= 85) && (TARGET_FREQUENCY <= 94))
			tap_amount = 29;
		else if ((TARGET_FREQUENCY >= 95) && (TARGET_FREQUENCY <= 109))
			tap_amount = 24;
		else if ((TARGET_FREQUENCY >= 110) && (TARGET_FREQUENCY <= 119))
			tap_amount = 42;
		else if ((TARGET_FREQUENCY >= 120) && (TARGET_FREQUENCY <= 134))
			tap_amount = 37;
		else if ((TARGET_FREQUENCY >= 135) && (TARGET_FREQUENCY <= 149))
			tap_amount = 32;
		else if ((TARGET_FREQUENCY >= 150) && (TARGET_FREQUENCY <= 174))
			tap_amount = 26;
		else if ((TARGET_FREQUENCY >= 175) && (TARGET_FREQUENCY <= 199))
			tap_amount = 21;
		else if ((TARGET_FREQUENCY >= 200) && (TARGET_FREQUENCY <= 224))
			tap_amount = 17;
		else if ((TARGET_FREQUENCY >= 225) && (TARGET_FREQUENCY <= 250))
			tap_amount = 14;

		start_tap_number_func =tap_amount;
	end
	endfunction


	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	//
	// Decision if the HDL-code is used for simulation or for
	// synthesis
	//
	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	function simulation_func;
		input [0:0]ignored;
	begin
		simulation_func = 0;
		  // synopsys translate_off
		simulation_func = 1;
		  // synopsys translate_on
	end
	endfunction


	function [1:0] dll_divide_factor_func;
		input [7:0] TARGET_FREQUENCY;
		integer divide;
	begin
		//dll_divide_factor_func = 0;
		divide = 0;
		if ((TARGET_FREQUENCY >= 12) && (TARGET_FREQUENCY <= 24))
			//dll_divide_factor_func = 3;//div 8
			divide = 3;//div 8
		else if ((TARGET_FREQUENCY >= 25) && (TARGET_FREQUENCY <= 54))
			//dll_divide_factor_func = 2;//div 4
			divide = 2;//div 4
		else if ((TARGET_FREQUENCY >= 55) && (TARGET_FREQUENCY <= 109))
			//dll_divide_factor_func = 1;//div 2
			divide = 1;//div 2
		else if ((TARGET_FREQUENCY >= 110) && (TARGET_FREQUENCY <= 250))
			//dll_divide_factor_func = 0;//div 1
			divide = 0;//div 1
		dll_divide_factor_func = divide;
	end
	endfunction

	function real get_input_period_func;
		input [7:0] TARGET_FREQUENCY;
		input [4:0] DLL_MULTIPLY;//: integer range 1 to 32)
		real final_period;//: real;
		real period;//: real;
		integer period_integer;//: integer;
	begin
		period = (1000.0/TARGET_FREQUENCY) * DLL_MULTIPLY;
		period_integer = period;//--avoid too many positions after decimal point
		final_period = period_integer;
	
		get_input_period_func = final_period;
	end
	endfunction

	function integer round_up_func;
		input real val;
		integer val_int;//: integer;
		real remainder;//: real;
	begin
		val_int = val;
		remainder = val - val_int;

		if (remainder < 0.0)
			round_up_func = val_int;
		else
			round_up_func = val_int + 1;
	end

	endfunction


	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	//
	// This implementation of the binary logarithm is based on the
	// code found here:
	// http://en.literateprograms.org/Logarithm_Function_%28Python%29
	//
	// For hardware-implemenation it has been simplified to less
	// accuracy.
	//
	//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	function real log_bin_func;
		input real val;

		localparam real epsilon = 1.0E-3;

		integer int_value;
		real val_int;

		real decfrac;
		real partial;

	begin
		int_value = 0;

		val_int = val;

		while (val_int < 1.0) begin
			int_value = int_value - 1;
			val_int = val_int * 2.0;
		end
		
		while (val_int >= 2.0) begin
			int_value = int_value + 1;
			val_int = val_int / 2.0;
		end
		
		decfrac = 0.0;
		partial = 0.5;
		val_int = val_int * val_int;
		
		while (partial > epsilon) begin
			if (val_int >= 2.0) begin
				decfrac = decfrac + partial;
				val_int = val_int / 2.0;
			end
			partial = partial / 2.0;
			val_int = val_int * val_int;
		end
		log_bin_func = int_value + decfrac;
	end 
	endfunction

//endmodule





























