///////////////////////////////////////////////////////////////////////////////
//
//    File Name:  clocking_test_iodll_r1.0.v
//      Version:  1.0
//         Date:  22.02.2008
//        Model:  
//
//      Company:  Xilinx, Inc.
//  Contributor:  Martin Kellermann
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2008 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
//
// Summary:
//  This module creates the a structure of two DCMs and two BUFGMUXes that allows
//  testing of various use-models of the IO-DLL.
//--------------------------------------------------------------------------------
`timescale 1ns / 1ps
module clocking_test_iodll #(
	parameter [7:0] DLL_MULTIPLY = 2,
	parameter [7:0] TARGET_FREQUENCY = 125
	)(
input REFCLK_P,
input REFCLK_N,
input RESET,
output REFCLK_O,
output REFCLK_IODLL,
output DCM_CLK,
input JITTERED_CLK,
output DCM_LOCK
);

wire  CLKIN_IBUFGDS;
wire  JITTER_CLKFX_BUF;
wire  CLKFX_SELECT;
wire  JITTER_CLK_OUT;
wire  JITTER_LOCK;
wire  MULT_CLKFX_BUF;
wire  MULT_SELECT;
wire  MULT_CLK_OUT;
wire  MULT_LOCK;

	function real get_input_period_func;
		input [7:0] TARGET_FREQUENCY;
		input [4:0] DLL_MULTIPLY;//: integer range 1 to 32)
		real final_period;//: real;
		real period;//: real;
		integer period_integer;//: integer;
	begin
		period = (1000.0/TARGET_FREQUENCY) * DLL_MULTIPLY;
		period_integer = period;//--avoid too many positions after decimal point
		final_period = period_integer;
	
		get_input_period_func = final_period;
	end
	endfunction



parameter VCC = 1'b 1;
parameter GND = 1'b 0;
parameter GNC_VEC = 16'b 0000000000000000;

  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //
  // Differential Clock-Input to FPGA
  //
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  IBUFGDS REFCLK_IBUFGDS(
      .I(REFCLK_P),
    .IB(REFCLK_N),
    .O(CLKIN_IBUFGDS));

  assign REFCLK_O = CLKIN_IBUFGDS;
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //
  // MUXing between unaltered reference-clock and jittered
  // CLKFX
  //
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  BUFGMUX CLKFX_BUFGMUX_INST(
      .I0(JITTER_CLKFX_BUF),
    .I1(CLKIN_IBUFGDS),
    .S(CLKFX_SELECT),
    .O(JITTER_CLK_OUT));

  assign CLKFX_SELECT =  ~JITTERED_CLK;
  assign REFCLK_IODLL = JITTER_CLK_OUT;
  generate if ((TARGET_FREQUENCY >= 130)) begin
      DCM_ADV #(
          .CLK_FEEDBACK("NONE"),
      .CLKDV_DIVIDE(2.0),
      .CLKFX_DIVIDE(32),
      .CLKFX_MULTIPLY(31),
      .CLKIN_DIVIDE_BY_2("FALSE"),
      .CLKIN_PERIOD(get_input_period_func(TARGET_FREQUENCY, DLL_MULTIPLY)),
      .CLKOUT_PHASE_SHIFT("NONE"),
      .DCM_AUTOCALIBRATION("TRUE"),
      .DCM_PERFORMANCE_MODE("MAX_SPEED"),
      .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),
      .DFS_FREQUENCY_MODE("LOW"),
      .DLL_FREQUENCY_MODE("LOW"),
      .DUTY_CYCLE_CORRECTION("TRUE"),
      .PHASE_SHIFT(0),
      .STARTUP_WAIT("FALSE"),
      .SIM_DEVICE("VIRTEX5"))
    JITTER_DCM(
          .CLKFB(GND),
      .CLKIN(CLKIN_IBUFGDS),
      .DADDR(GNC_VEC[6:0] ),
      .DCLK(GND),
      .DEN(GND),
      .DI(GNC_VEC),
      .DWE(GND),
      .PSCLK(GND),
      .PSEN(GND),
      .PSINCDEC(GND),
      .RST(RESET),
      .CLKDV(),
      .CLKFX(JITTER_CLKFX_BUF),
      .CLKFX180(),
      .CLK0(),
      .CLK2X(),
      .CLK2X180(),
      .CLK90(),
      .CLK180(),
      .CLK270(),
      .DO(),
      .DRDY(),
      .LOCKED(JITTER_LOCK),
      .PSDONE());

  end
  endgenerate
  generate if ((TARGET_FREQUENCY >= 40) && (TARGET_FREQUENCY < 130)) begin
      DCM_ADV #(
          .CLK_FEEDBACK("NONE"),
      .CLKDV_DIVIDE(2.0),
      .CLKFX_DIVIDE(32),
      .CLKFX_MULTIPLY(31),
      .CLKIN_DIVIDE_BY_2("FALSE"),
      .CLKIN_PERIOD(7),
      //get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
      .CLKOUT_PHASE_SHIFT("NONE"),
      .DCM_AUTOCALIBRATION("TRUE"),
      .DCM_PERFORMANCE_MODE("MAX_SPEED"),
      .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),
      .DFS_FREQUENCY_MODE("LOW"),
      .DLL_FREQUENCY_MODE("LOW"),
      .DUTY_CYCLE_CORRECTION("TRUE"),
      .PHASE_SHIFT(0),
      .STARTUP_WAIT("FALSE"),
      .SIM_DEVICE("VIRTEX5"))
    JITTER_DCM(
          .CLKFB(GND),
      .CLKIN(CLKIN_IBUFGDS),
      .DADDR(GNC_VEC[6:0] ),
      .DCLK(GND),
      .DEN(GND),
      .DI(GNC_VEC),
      .DWE(GND),
      .PSCLK(GND),
      .PSEN(GND),
      .PSINCDEC(GND),
      .RST(RESET),
      .CLKDV(),
      .CLKFX(JITTER_CLKFX_BUF),
      .CLKFX180(),
      .CLK0(),
      .CLK2X(),
      .CLK2X180(),
      .CLK90(),
      .CLK180(),
      .CLK270(),
      .DO(),
      .DRDY(),
      .LOCKED(JITTER_LOCK),
      .PSDONE());

  end
  endgenerate
  generate if (TARGET_FREQUENCY < 40) begin
      DCM_ADV #(
          .CLK_FEEDBACK("NONE"),
      .CLKDV_DIVIDE(2.0),
      .CLKFX_DIVIDE(32),
      .CLKFX_MULTIPLY(31),
      .CLKIN_DIVIDE_BY_2("FALSE"),
      .CLKIN_PERIOD(7),
      //get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
      .CLKOUT_PHASE_SHIFT("NONE"),
      .DCM_AUTOCALIBRATION("TRUE"),
      .DCM_PERFORMANCE_MODE("MAX_RANGE"),
      .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),
      .DFS_FREQUENCY_MODE("LOW"),
      .DLL_FREQUENCY_MODE("LOW"),
      .DUTY_CYCLE_CORRECTION("TRUE"),
      .PHASE_SHIFT(0),
      .STARTUP_WAIT("FALSE"),
      .SIM_DEVICE("VIRTEX5"))
    JITTER_DCM(
          .CLKFB(GND),
      .CLKIN(CLKIN_IBUFGDS),
      .DADDR(GNC_VEC[6:0] ),
      .DCLK(GND),
      .DEN(GND),
      .DI(GNC_VEC),
      .DWE(GND),
      .PSCLK(GND),
      .PSEN(GND),
      .PSINCDEC(GND),
      .RST(RESET),
      .CLKDV(),
      .CLKFX(JITTER_CLKFX_BUF),
      .CLKFX180(),
      .CLK0(),
      .CLK2X(),
      .CLK2X180(),
      .CLK90(),
      .CLK180(),
      .CLK270(),
      .DO(),
      .DRDY(),
      .LOCKED(JITTER_LOCK),
      .PSDONE());

  end
  endgenerate
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  //
  // MUXing between multiplied DCM-clock and jittered CLKFX from
  // previous DCM
  //
  //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  BUFGMUX CLKFX_BUFGMUX_CTRL_INST(
      .I0(MULT_CLKFX_BUF),
    .I1(JITTER_CLK_OUT),
    .S(MULT_SELECT),
    .O(MULT_CLK_OUT));

  assign MULT_SELECT = JITTERED_CLK;
  assign DCM_CLK = MULT_CLK_OUT;

  generate if (TARGET_FREQUENCY >= 110) begin
//      generate if (DLL_MULTIPLY == 1) begin
      if (DLL_MULTIPLY == 1) begin
          // CLKFX_MULTIPLY=1 not allowed on DCM, hence use CLKFX_MULTIPLY and CLKFX_DIVIDE = 2
      DCM_ADV #(
              .CLK_FEEDBACK("NONE"),
        .CLKDV_DIVIDE(2.0),
        .CLKFX_DIVIDE(2),
        .CLKFX_MULTIPLY(2),
        .CLKIN_DIVIDE_BY_2("FALSE"),
        .CLKIN_PERIOD(7),
        //get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
        .CLKOUT_PHASE_SHIFT("NONE"),
        .DCM_AUTOCALIBRATION("TRUE"),
        .DCM_PERFORMANCE_MODE("MAX_SPEED"),
        .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),
        .DFS_FREQUENCY_MODE("HIGH"),
        .DLL_FREQUENCY_MODE("LOW"),
        .DUTY_CYCLE_CORRECTION("TRUE"),
        .PHASE_SHIFT(0),
        .STARTUP_WAIT("FALSE"),
        .SIM_DEVICE("VIRTEX5"))
      MULTIPLY_DCM_HIGH(
              .CLKFB(GND),
        .CLKIN(JITTER_CLK_OUT),
        .DADDR(GNC_VEC[6:0] ),
        .DCLK(GND),
        .DEN(GND),
        .DI(GNC_VEC),
        .DWE(GND),
        .PSCLK(GND),
        .PSEN(GND),
        .PSINCDEC(GND),
        .RST(RESET),
        .CLKDV(),
        .CLKFX(MULT_CLKFX_BUF),
        .CLKFX180(),
        .CLK0(),
        .CLK2X(),
        .CLK2X180(),
        .CLK90(),
        .CLK180(),
        .CLK270(),
        .DO(),
        .DRDY(),
        .LOCKED(MULT_LOCK),
        .PSDONE());

    end
//    endgenerate
//    generate if (DLL_MULTIPLY > 1) begin
    if (DLL_MULTIPLY > 1) begin
          DCM_ADV #(
              .CLK_FEEDBACK("NONE"),
        .CLKDV_DIVIDE(2.0),
        .CLKFX_DIVIDE(1),
        .CLKFX_MULTIPLY(DLL_MULTIPLY),
        .CLKIN_DIVIDE_BY_2("FALSE"),
        .CLKIN_PERIOD(7),
        //get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
        .CLKOUT_PHASE_SHIFT("NONE"),
        .DCM_AUTOCALIBRATION("TRUE"),
        .DCM_PERFORMANCE_MODE("MAX_SPEED"),
        .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),
        .DFS_FREQUENCY_MODE("HIGH"),
        .DLL_FREQUENCY_MODE("LOW"),
        .DUTY_CYCLE_CORRECTION("TRUE"),
        .PHASE_SHIFT(0),
        .STARTUP_WAIT("FALSE"),
        .SIM_DEVICE("VIRTEX5"))
      MULTIPLY_DCM_HIGH(
              .CLKFB(GND),
        .CLKIN(JITTER_CLK_OUT),
        .DADDR(GNC_VEC[6:0] ),
        .DCLK(GND),
        .DEN(GND),
        .DI(GNC_VEC),
        .DWE(GND),
        .PSCLK(GND),
        .PSEN(GND),
        .PSINCDEC(GND),
        .RST(RESET),
        .CLKDV(),
        .CLKFX(MULT_CLKFX_BUF),
        .CLKFX180(),
        .CLK0(),
        .CLK2X(),
        .CLK2X180(),
        .CLK90(),
        .CLK180(),
        .CLK270(),
        .DO(),
        .DRDY(),
        .LOCKED(MULT_LOCK),
        .PSDONE());

    end
    //endgenerate
  end
  endgenerate
  generate if (TARGET_FREQUENCY < 110) begin
//      generate if (DLL_MULTIPLY == 1) begin
      if (DLL_MULTIPLY == 1) begin
          // CLKFX_MULTIPLY=1 not allowed on DCM, hence use CLKFX_MULTIPLY and CLKFX_DIVIDE = 2
      DCM_ADV #(
              .CLK_FEEDBACK("NONE"),
        .CLKDV_DIVIDE(2.0),
        .CLKFX_DIVIDE(2),
        .CLKFX_MULTIPLY(2),
        .CLKIN_DIVIDE_BY_2("FALSE"),
        .CLKIN_PERIOD(7),
        //get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
        .CLKOUT_PHASE_SHIFT("NONE"),
        .DCM_AUTOCALIBRATION("TRUE"),
        .DCM_PERFORMANCE_MODE("MAX_SPEED"),
        .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),
        .DFS_FREQUENCY_MODE("LOW"),
        .DLL_FREQUENCY_MODE("LOW"),
        .DUTY_CYCLE_CORRECTION("TRUE"),
        .PHASE_SHIFT(0),
        .STARTUP_WAIT("FALSE"),
        .SIM_DEVICE("VIRTEX5"))
      MULTIPLY_DCM_LOW(
              .CLKFB(GND),
        .CLKIN(JITTER_CLK_OUT),
        .DADDR(GNC_VEC[6:0] ),
        .DCLK(GND),
        .DEN(GND),
        .DI(GNC_VEC),
        .DWE(GND),
        .PSCLK(GND),
        .PSEN(GND),
        .PSINCDEC(GND),
        .RST(RESET),
        .CLKDV(),
        .CLKFX(MULT_CLKFX_BUF),
        .CLKFX180(),
        .CLK0(),
        .CLK2X(),
        .CLK2X180(),
        .CLK90(),
        .CLK180(),
        .CLK270(),
        .DO(),
        .DRDY(),
        .LOCKED(MULT_LOCK),
        .PSDONE());

    end
    //endgenerate
    //generate if (DLL_MULTIPLY > 1) begin
    if (DLL_MULTIPLY > 1) begin
          DCM_ADV #(
              .CLK_FEEDBACK("NONE"),
        .CLKDV_DIVIDE(2.0),
        .CLKFX_DIVIDE(1),
        .CLKFX_MULTIPLY(DLL_MULTIPLY),
        .CLKIN_DIVIDE_BY_2("FALSE"),
        .CLKIN_PERIOD(7),//get_input_period_func(.TARGET_FREQUENCY(TARGET_FREQUENCY), .DLL_MULTIPLY(DLL_MULTIPLY)),
        .CLKOUT_PHASE_SHIFT("NONE"),
        .DCM_AUTOCALIBRATION("TRUE"),
        .DCM_PERFORMANCE_MODE("MAX_SPEED"),
        .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),
        .DFS_FREQUENCY_MODE("LOW"),
        .DLL_FREQUENCY_MODE("LOW"),
        .DUTY_CYCLE_CORRECTION("TRUE"),
        .PHASE_SHIFT(0),
        .STARTUP_WAIT("FALSE"),
        .SIM_DEVICE("VIRTEX5"))
      MULTIPLY_DCM_LOW(
              .CLKFB(GND),
        .CLKIN(JITTER_CLK_OUT),
        .DADDR(GNC_VEC[6:0] ),
        .DCLK(GND),
        .DEN(GND),
        .DI(GNC_VEC),
        .DWE(GND),
        .PSCLK(GND),
        .PSEN(GND),
        .PSINCDEC(GND),
        .RST(RESET),
        .CLKDV(),
        .CLKFX(MULT_CLKFX_BUF),
        .CLKFX180(),
        .CLK0(),
        .CLK2X(),
        .CLK2X180(),
        .CLK90(),
        .CLK180(),
        .CLK270(),
        .DO(),
        .DRDY(),
        .LOCKED(MULT_LOCK),
        .PSDONE());

    end
    //endgenerate
  end
  endgenerate
  
    assign DCM_LOCK = JITTERED_CLK ? JITTER_LOCK : MULT_LOCK;

endmodule
