//
///////////////////////////////////////////////////////////////////////////////
//
//    File Name:  io_dll_top_r1.0.vhd
//      Version:  1.0
//         Date:  22.02.2008
//        Model:  
//
//      Company:  Xilinx, Inc.
//  Contributor:  Martin Kellermann
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2008 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
// 
// Summary:
//
//--------------------------------------------------------------------------------
`timescale 1ps / 1ps
module io_dll_top #(
	parameter [7:0] DLL_MULTIPLY = 2,
	parameter [7:0] TARGET_FREQUENCY = 125
	)
(
input REFCLK_P,
input REFCLK_N,
input CLK_200M_P,
input CLK_200M_N,
input IDLAYCTRLRESET,
output CLKDLL_OUT_SMA_P,
output CLKDLL_OUT_SMA_N,
output CLKDLL_OUT_PIN_P,
output CLKDLL_OUT_PIN_N,
output CLKREF_OUT_P,
output CLKREF_OUT_N,
output IDLAYRDY,
output ERROR,
output[5:0] TOGGLE,
output LOCK_DCM,
output LOCK_DLL,
input JITTERED_CLOCK,
input RESET
);


wire  rdy;
wire  refclk_ibuf;
wire  refclk;
wire  dcm_clkfx;
wire  dcm_lock;
wire  out_clk;
wire  dll_clk;
wire  dll_lock;
wire [4:0] prbs_gen;
wire [4:0] prbs_check;
wire  jittered_clock_int;
wire  error_int;
wire  reset_int;
wire  dly_ctrl_reset;
wire  dcm_wr_lock;
wire  dll_rd_lock;
wire  empty;
wire  full;
wire  overflow;
wire  underflow;
wire [12:0] rd_data_count;
wire [12:0] wr_data_count;
reg [27:0] tog_cnt;//=0;
reg [27:0] tog_cnt2;//=0;

	initial begin
		tog_cnt = 0;
		tog_cnt2 = 0;
	end

//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//
// Basic Components
//
//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


  DLY_CTRL IDELAY_CTRLER(
      .CLK_200M_P(CLK_200M_P),
    .CLK_200M_N(CLK_200M_N),
    .RESET(dly_ctrl_reset),
    .RDY(rdy));

  assign IDLAYRDY = rdy;
  reset_cleaner CLEAN_INPUTS(
      .RESET(RESET),
    .RESET_DLY_CTRL(IDLAYCTRLRESET),
    .JITTERED_CLOCK(JITTERED_CLOCK),
    .RESET_CLEAN(reset_int),
    .RESET_DLY_CTRL_CLEAN(dly_ctrl_reset),
    .JITTERED_CLOCK_CLEAN(jittered_clock_int),
    .CLK(refclk_ibuf));

  clocking_test_iodll #(
		.TARGET_FREQUENCY(TARGET_FREQUENCY),
    .DLL_MULTIPLY(DLL_MULTIPLY))
  std_clocks(
		.REFCLK_P(REFCLK_P),
    .REFCLK_N(REFCLK_N),
    .RESET(reset_int),
    .REFCLK_O(refclk_ibuf),
    .REFCLK_IODLL(refclk),
    .DCM_CLK(dcm_clkfx),
    .JITTERED_CLK(jittered_clock_int),
    .DCM_LOCK(dcm_lock));

  assign LOCK_DCM = dcm_lock;

  assign LOCK_DLL = dll_lock;
  clk_out SMA_forward_dllclk(
		.CLK(dll_clk),
    .CLK_O_P(CLKDLL_OUT_SMA_P),
    .CLK_O_N(CLKDLL_OUT_SMA_N));

  clk_out PIN_forward_dllclk(
      .CLK(dll_clk),
    .CLK_O_P(CLKDLL_OUT_PIN_P),
    .CLK_O_N(CLKDLL_OUT_PIN_N));

  clk_out forward_refclk(
      .CLK(refclk),
    .CLK_O_P(CLKREF_OUT_P),
    .CLK_O_N(CLKREF_OUT_N));

//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//	--
//	-- End Basic Components
//	--
//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//	--
//	-- IO-DLL
//	--
//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  io_dll #(
		.CLOCK_BUFFER("BUFG"),
		.DLL_MULTIPLY(DLL_MULTIPLY),
    .TARGET_FREQUENCY(TARGET_FREQUENCY))
  iodll(
		.REFCLK(refclk),
    .DLLCLK(dll_clk),
    .LOCKED(dll_lock),
    .RESET(reset_int));
//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//	--
//	-- End IO-DLL
//	--
//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//	--
//	-- Checkers for 0 ppm offset
//	--
//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  prbs_5_2v7_x7_x6 prbs_generator(
      .CE(dcm_lock),
    .RST(reset_int),
    .CLK(dcm_clkfx),
    .Q(prbs_gen));
  prbs_5_2v7_x7_x6_check checker(
      .CE(dll_rd_lock),
    .RST(reset_int),
    .CLK(dll_clk),
    .ERROR(error_int),
    .D(prbs_check));

//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
//	--
//	-- End Checkers for 0 ppm offset
//	--
//	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


  wrrd_fifo start_FIFO(
      .CLK(refclk),
    .RESET(reset_int),
    .WR_LOCK(dcm_lock),
    .RD_LOCK(dll_lock),
    .WR_LOCK_O(dcm_wr_lock),
    .RD_LOCK_O(dll_rd_lock));


  x5_fifo_bram decouple_FIFO(
    .din(prbs_gen),
    .wr_clk(dcm_clkfx),
    .wr_en(dcm_wr_lock),
    .rd_clk(dll_clk),
    .rd_en(dll_rd_lock),
    .dout(prbs_check),
    .rst(reset_int),
    .rd_data_count(rd_data_count),
    .wr_data_count(wr_data_count),
    .empty(empty),
    .full(full),
    .overflow(overflow),
    .underflow(underflow));

  assign ERROR = error_int;
  always @(posedge dll_clk) begin
    if(reset_int == 1'b 1) begin
      tog_cnt <= {28{1'b1}};
    end
    else begin
      tog_cnt <= tog_cnt + 1'b 1;
    end
  end

  assign TOGGLE[0]  = tog_cnt[27] ;
  always @(posedge refclk) begin
    if(reset_int == 1'b 1) begin
      tog_cnt2 <= {28{1'b1}};
    end
    else begin
      tog_cnt2 <= tog_cnt2 + 1'b 1;
    end
  end

  assign TOGGLE[1]  = tog_cnt2[27] ;
  assign TOGGLE[5:2]  = {4{1'b0}};

endmodule
