//
///////////////////////////////////////////////////////////////////////////////
//
//    File Name:  prbs_5_2v7_x7_x6_check_r1.0.v
//      Version:  1.0
//         Date:  22.02.2008
//        Model:  
//
//      Company:  Xilinx, Inc.
//  Contributor:  Martin Kellermann
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2008 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
// 
//
// 5-bit Parallel PRBS Checker for 2^7-1 with feedback of X7 + X6
//
// Next expected PRBS-pattern is generated from incoming data
//
//
//--------------------------------------------------------------------------------
`timescale 1ns / 1ps
module prbs_5_2v7_x7_x6_check(
input CE,
input RST,
input CLK,
output ERROR,
input [4:0] D
);

	reg error_int;
	assign ERROR = error_int;

	reg [12:1] next_PRBS;//=0;
	wire [4:0] PRBS_current;

	reg  [4:0] test_PRBS;
	reg  [4:0] incoming_PRBS;//=0;

	initial begin
		error_int = 0;		
		next_PRBS = 0;
		incoming_PRBS = 0;
	end



  assign PRBS_current = D;
  
	always @(posedge CLK) begin
    if((RST == 1'b 1)) begin
      next_PRBS[12:1]  <= 12'b 111111100000;
    end
    else if((CE == 1'b 1)) begin
      next_PRBS[12]  <= next_PRBS[7] ;
      next_PRBS[11]  <= next_PRBS[6] ;
      next_PRBS[10]  <= next_PRBS[5] ;
      next_PRBS[9]  <= next_PRBS[4] ;
      next_PRBS[8]  <= next_PRBS[3] ;
      next_PRBS[7]  <= next_PRBS[2] ;
      next_PRBS[6]  <= next_PRBS[1] ;
      next_PRBS[5]  <= next_PRBS[6]  ^ next_PRBS[7] ;
      next_PRBS[4]  <= next_PRBS[5]  ^ next_PRBS[6] ;
      next_PRBS[3]  <= PRBS_current[4]  ^ next_PRBS[5] ;
      next_PRBS[2]  <= PRBS_current[3]  ^ PRBS_current[4] ;
      next_PRBS[1]  <= PRBS_current[2]  ^ PRBS_current[3] ;
      test_PRBS[4:1]  <= next_PRBS[4:1] ;
      incoming_PRBS <= PRBS_current;
    end
  end

	always @(next_PRBS[5])
	begin	
		test_PRBS[0]<=next_PRBS[5];
	end
	
  always @(posedge CLK) begin
    if(RST == 1'b 1) begin
      error_int <= 1'b 0;
    end else begin
      if(incoming_PRBS != test_PRBS) begin
        error_int <= 1'b 1;
      end
      else begin
        error_int <= 1'b 0;
      end
    end
  end


endmodule
