//
///////////////////////////////////////////////////////////////////////////////
//
//    File Name:  reset_cleaner_r1.0.v
//      Version:  1.0
//         Date:  22.02.2008
//        Model:
//
//      Company:  Xilinx, Inc.
//  Contributor:  Martin Kellermann
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2008 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
//
// Summary:
//  This module is used to clean the asynchronuous inputs and avoid issues
//  inside the FPGA because of runt pulses and meta-stability.
//--------------------------------------------------------------------------------
`timescale 1ns / 1ps
module reset_cleaner(
input RESET,
input RESET_DLY_CTRL,
input JITTERED_CLOCK,
output RESET_CLEAN,
output RESET_DLY_CTRL_CLEAN,
output JITTERED_CLOCK_CLEAN,
input CLK
);

reg [2:0] reset_int;//=0;
reg [2:0] reset_dly_ctrl_int;//=0;
reg [2:0] jittered_clock_int;//=0;

reg reset_clean_int;
reg reset_dly_ctrl_clean_int;
reg jittered_clock_clean_int;

assign RESET_CLEAN = reset_clean_int;
assign RESET_DLY_CTRL_CLEAN = reset_dly_ctrl_clean_int;
assign JITTERED_CLOCK_CLEAN = jittered_clock_clean_int;


initial
   begin : init_process
   reset_int = 3'b 000;
   reset_dly_ctrl_int = 3'b 000;
   jittered_clock_int = 3'b 000;
   end

  always @(posedge CLK) begin
    reset_int[2]  <= RESET;
    reset_dly_ctrl_int[2]  <= RESET_DLY_CTRL;
    jittered_clock_int[2]  <= JITTERED_CLOCK;
    
		reset_int[1:0]  <= reset_int[2:1] ;
    reset_dly_ctrl_int[1:0]  <= reset_dly_ctrl_int[2:1] ;
    jittered_clock_int[1:0]  <= jittered_clock_int[2:1] ;
    
		reset_clean_int <= reset_int[2]  & reset_int[1]  & reset_int[0] ;
    reset_dly_ctrl_int <= reset_dly_ctrl_int[2]  & reset_dly_ctrl_int[1]  & reset_dly_ctrl_int[0] ;
    jittered_clock_clean_int <= jittered_clock_int[2]  & jittered_clock_int[1]  & jittered_clock_int[0] ;
  end


endmodule
