//
///////////////////////////////////////////////////////////////////////////////
//
//    File Name:  wrrd_fifo_r1.0.v
//      Version:  1.0
//         Date:  22.02.2008
//        Model:  
//
//      Company:  Xilinx, Inc.
//  Contributor:  Martin Kellermann
//
//   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
//                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
//                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
//                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
//                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
//                APPLICATION OR STANDARD, XILINX IS MAKING NO
//                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
//                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
//                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
//                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
//                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
//                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
//                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
//                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
//                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//                PURPOSE.
//
//                (c) Copyright 2008 Xilinx, Inc.
//                All rights reserved.
//
///////////////////////////////////////////////////////////////////////////////
// 
// Summary:
//	This module takes the two lock-signals from the Write- and Read-domain
//	of the checking-FIFO and creates a proper timing between them.
//	This allows that the FIFO can start at some mid-point after both clock-
//	generators are locked and avoids starting at the high- or low limit of
//	the FIFO.
//
//--------------------------------------------------------------------------------
`timescale 1ns / 1ps
module wrrd_fifo #(parameter FIFO_DEPTH = 128)(
input CLK,
input RESET,
input WR_LOCK,
input RD_LOCK,
output WR_LOCK_O,
output RD_LOCK_O
);

//parameter FIFO_DEPTH = 128;




	function integer depth_check_func;
	input integer FIFO_DEPTH;
	begin
		if(FIFO_DEPTH > 200)
			depth_check_func = 100;
		else
			depth_check_func = FIFO_DEPTH / 2;
		end
	endfunction

parameter FIFO_START_POINT = depth_check_func(FIFO_DEPTH);

integer clk_cnt;

reg [2:0] lock_wr;// = 0;
reg [2:0] lock_rd;// = 0;
reg  both_locked;// = 0;

reg wr_lock_int;// = 0;
reg rd_lock_int;// = 0;

assign WR_LOCK_O = wr_lock_int;
assign RD_LOCK_O = rd_lock_int;

  initial begin
    both_locked = 0;
    lock_wr = 0;
		lock_rd = 0;
   end

  always @(posedge CLK) begin
    if(RESET == 1'b 1) begin
      lock_wr <= {3{1'b0}};
      lock_rd <= {3{1'b0}};
    end
    else begin
      lock_wr[2]  <= WR_LOCK;
      lock_wr[1:0]  <= lock_wr[2:1] ;
      lock_rd[2]  <= RD_LOCK;
      lock_rd[1:0]  <= lock_rd[2:1] ;
    end
  end

  always @(posedge CLK) begin
    if((lock_wr[0]  == 1'b 1) && (lock_rd[0]  == 1'b 1)) begin
      both_locked <= 1'b 1;
    end
    else begin
      both_locked <= 1'b 0;
    end
  end

  always @(posedge CLK) begin
    if(both_locked == 1'b 1) begin
      wr_lock_int <= 1'b 1;
      if(clk_cnt < (FIFO_START_POINT - 1)) begin
        clk_cnt <= clk_cnt + 1;
      end
    end
    else begin
      wr_lock_int <= 1'b 0;
      clk_cnt <= 0;
    end
    if(clk_cnt == (FIFO_START_POINT - 1)) begin
      rd_lock_int <= 1'b 1;
    end
    else begin
      rd_lock_int <= 1'b 0;
    end
  end


endmodule
