--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  dll_config_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2007 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:
--  This module serves as a wrapper for the adjustable oscillator and provides
--  some hooks for debugging. In the normal use those signals are not used but
--  may be monitored using e.g. ChipScope.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


library work;
use work.pkg_dll_reco.all;

entity dll_config is
		Generic (
			CLOCK_BUFFER : string := "BUFR";
			TARGET_FREQUENCY: integer range 12 to 250:=125	--nominal frequency in full MHz
			);
		Port ( CLK_200M : in  STD_LOGIC;
					 FASTER : in  STD_LOGIC;
					 SLOWER : in  STD_LOGIC;
					 STOP_UPDATE: in std_logic;
					 DLL_CLK : out  STD_LOGIC;
					 RESET : in  STD_LOGIC;
					 RESET_DLY_CTRL : in  STD_LOGIC;
					 DLY_CTRL_RDY : out  STD_LOGIC;
					 CTRL_CLK : in  STD_LOGIC);
end dll_config;

architecture Behavioral of dll_config is
	COMPONENT dll_osc
	GENERIC(
		CLOCK_BUFFER : string := "BUFR";
		TAP_AMOUNT : integer range 1 to 63:=23;
		DLL_DIV: integer range 0 to 3:=0);
	PORT(
		RESET : IN std_logic;
		RESET_DLY_CTRL : IN std_logic;
		CLK_200M : IN std_logic;
		CTRL_CLK : IN std_logic;
		STOP_UPDATE: in std_logic;
		FASTER : IN std_logic;
		SLOWER : IN std_logic;
		ERR_TOO_FAST: OUT std_logic;
		ERR_TOO_SLOW: OUT std_logic;
		DLL_CLK : OUT std_logic;
		DLY_CTRL_RDY : OUT std_logic
		);
	END COMPONENT;

	signal err_too_fast, err_too_slow: std_logic;
	
begin
	osc_core: dll_osc 
	GENERIC MAP(
		CLOCK_BUFFER => CLOCK_BUFFER,
		TAP_AMOUNT => start_tap_number_func(TARGET_FREQUENCY),
		DLL_DIV => dll_divide_factor_func(TARGET_FREQUENCY)
	)
	PORT MAP(
		DLL_CLK => DLL_CLK,
		RESET => RESET,
		RESET_DLY_CTRL => RESET_DLY_CTRL,
		STOP_UPDATE => STOP_UPDATE,
		DLY_CTRL_RDY => DLY_CTRL_RDY,
		CLK_200M => CLK_200M,
		CTRL_CLK => CTRL_CLK,
		ERR_TOO_FAST => err_too_fast,
		ERR_TOO_SLOW => err_too_slow,
		FASTER => FASTER,
		SLOWER => SLOWER
	);
	
end Behavioral;

