--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  dll_osc_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2008 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:
--	This module is the core of the DLL built out of the IODELAY. The IODELAY is
--	looped back and inverted onto itself, generating an oscillation.
--	The period of the oscillaton is 2 x (number_of_taps * 80 ps)
--
--	Example (ignoring routing-delay, see package-file for detailled values): 
--	* 100 MHz: number_of_taps = 63
--	* 200 MHz: number_of_taps = 31
--	* 250 MHz: number_of_taps = 25
--	* 125 MHz: number_of_taps = 25, div2 
--
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.pkg_dll_reco.all;

library UNISIM;
use UNISIM.VComponents.all;

entity dll_osc is
		Generic (
			CLOCK_BUFFER : string := "BUFR";
			TAP_AMOUNT : integer range 1 to 63:=31;
			DLL_DIV: integer range 0 to 3:=0);
		Port (	DLL_CLK : out  STD_LOGIC;
					 RESET : in  STD_LOGIC;
					 RESET_DLY_CTRL : in  STD_LOGIC;
					 STOP_UPDATE: in std_logic;
					 DLY_CTRL_RDY : out STD_LOGIC:='0';
					 CLK_200M : in STD_LOGIC;
					 CTRL_CLK: in STD_LOGIC;
					 ERR_TOO_FAST: out STD_LOGIC:='0';
					 ERR_TOO_SLOW: out STD_LOGIC:='0';
					 FASTER : in  STD_LOGIC;
					 SLOWER : in  STD_LOGIC);
end dll_osc;

architecture Behavioral of dll_osc is
	component IDELAYCTRL port (
		RDY : out std_logic;
		REFCLK : in std_logic;
		RST : in std_logic);
	end component;
	component BUFG port (
		O : out std_logic;
		I : in std_logic);
	end component;
	component BUFR port (
		O : out std_logic;
		CLR : in std_logic;
		CE : in std_logic;
		I : in std_logic);
	end component;
	component BUF port (
		O : out std_logic;
		I : in std_logic);
	end component;


	component IODELAY
		generic (
			DELAY_SRC : string := "DATAIN";
			HIGH_PERFORMANCE_MODE : boolean := true;
			IDELAY_TYPE : string := "VARIABLE";
			IDELAY_VALUE : integer := 0;
			ODELAY_VALUE : integer := 0;
			REFCLK_FREQUENCY : real := 200.0
		);
		port(
			DATAOUT : out std_ulogic;
			C : in std_ulogic;
			CE : in std_ulogic;
			DATAIN : in std_ulogic;
			IDATAIN : in std_ulogic;
			INC : in std_ulogic;
			ODATAIN : in std_ulogic;
			RST : in std_ulogic;
			T : in std_ulogic
		);
	end component;
	signal clk_int: std_logic:='0';
	signal clk_int_not: std_logic:='0';
	signal clk_int_not_dly: std_logic:='0';
	signal dll_clk_int: std_logic;
	signal inc, adj_ce: std_logic:='0';
	signal inc_ctrl, adj_ce_ctrl: std_logic:='0';
		
	signal div_cnt: std_logic_vector(2 downto 0):=(others=>'0');
	
	signal reset_cnt: std_logic_vector(1 downto 0):=(others=>'0');
	signal clean_pipe: std_logic:='1';
	signal tap_cnt: integer range 0 to 63:=TAP_AMOUNT;
	attribute keep: string;
	attribute keep of tap_cnt: signal is "true";
	attribute keep of dll_clk_int: signal is "true";
	attribute keep of DLL_CLK: signal is "true";
	
	attribute buffer_type: string;
	attribute buffer_type of dll_clk_int: signal is "NONE";
begin

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	--Local IDELAYCTRL, can be disabled in package
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
		IDELAYCTRL_gen: if LOCAL_IDELAYCTRL= true generate
		dlyctrl: IDELAYCTRL port map(
			RDY => DLY_CTRL_RDY,
			REFCLK => CLK_200M,
			RST => RESET_DLY_CTRL);
	end generate;

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- The beating heart of the system, the oscillator
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	osc:IODELAY
		generic map(
			IDELAY_VALUE => TAP_AMOUNT)
		port map(
			DATAOUT => clk_int,
			C => CTRL_CLK,
			CE => adj_ce_ctrl,
			DATAIN => clk_int_not_dly,
			IDATAIN =>'0',
			INC => inc_ctrl,
			ODATAIN =>'0',
			RST => RESET,
			T =>'0'
		);
		
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	--Divider-stages for the output of the DLL-oscillator
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	process(clk_int)
	begin
		if rising_edge(clk_int) then
			div_cnt<=div_cnt +'1';		
		end if;
	end process;
	clk_out_div1:if DLL_DIV=0 generate
		dll_clk_int <= clk_int;
	end generate;
	clk_out_div2:if DLL_DIV=1 generate
		dll_clk_int <= div_cnt(0);
	end generate;
	clk_out_div4:if DLL_DIV=2 generate
		dll_clk_int <= div_cnt(1);
	end generate;
	clk_out_div8:if DLL_DIV=3 generate
		dll_clk_int <= div_cnt(2);
	end generate;
		
	BUFG_usage: if CLOCK_BUFFER = "BUFG" generate
		DLL_BUFG: bufg port map(
			I => dll_clk_int,
			O =>DLL_CLK);
	end generate;
	BUFR_usage: if CLOCK_BUFFER = "BUFR" generate
		DLL_BUFR: bufr port map(
			I => dll_clk_int,
			CLR => '0',
			CE => '1',
			O =>DLL_CLK);
	end generate;
	local_usage: if CLOCK_BUFFER = "BUF" generate
		DLL_BUF: buf port map(
			I => dll_clk_int,
			O =>DLL_CLK);
	end generate;



	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- Cleaning of the delay-pipe for simulation, not neccessary 
	-- in hardware
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	process
	begin
		--This process is only used to start up the oscillation in simulation
		--by avoiding having a permanently inverted 'X' in the IODELAY.
		--During implementation the 'Latch' is stripped out and does not appear.
		if simulation_func then
			--pragma translate_off
			if now < 2 * tap_amount * 80 ps then
				clk_int_not<='1';
			else
				clk_int_not<= not clk_int;
			end if;
			--pragma translate_on
		else			
			clk_int_not<= not clk_int;
		end if;
		--500 ps wait-time to model the real delay on this net inside the FPGA
		--and to keep the simulation fast.

		--pragma translate_off
			if now < 2 * tap_amount * 80 ps then
				wait for 500 ps;
			else
				wait until clk_int'event;
			end if;

		--pragma translate_on
	end process;

	-- simulated delay of the FPGA-routing which has a big effect on the freqeuncy:
	clk_int_not_dly <= transport clk_int_not after 1 ns; 
	
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--	
	--Translate the FASTER-/SLOWER command into signals for
	--the IODELAY
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	process(CTRL_CLK)
		variable adj: std_logic_vector(1 downto 0);
	begin
		adj:= FASTER & SLOWER;
		if rising_edge(CTRL_CLK) then
			if RESET='1' then
				inc<='0';
				adj_ce<='0';
			elsif STOP_UPDATE ='0' then
				inc<='0';
				adj_ce<='0';
				case (adj) is
					when "01" =>
						inc <='1';
						adj_ce  <='1';
					when "10" =>
						inc <='0';
						adj_ce  <='1';
					when others => 
						inc <='0';
						adj_ce  <='0';
				
				end case;
			end if;
		end if;
	end process;

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	--Filter the IODELAY-commands to avoid getting too slow or fast:
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	process(CTRL_CLK)
	begin
		if rising_edge(CTRL_CLK) then
			if RESET = '1' then
				ERR_TOO_FAST<='0';
				ERR_TOO_SLOW<='0';
				
			elsif adj_ce ='1' then
				ERR_TOO_FAST<='0';
				ERR_TOO_SLOW<='0';

				if inc='1' then
					--incrementing the number of taps => f(dll)getting SLOWER
					if tap_cnt < MAX_TAP_CNT then
						inc_ctrl<=inc;
						adj_ce_ctrl<='1';
					else	
						ERR_TOO_SLOW<='1';
						report "DLL IODELAY getting too slow!" severity error;
					end if;
				else
					--decrementing the number of taps => f(dll)getting FASTER
					inc_ctrl<='0';
					if tap_cnt >= MIN_TAP_CNT then
						adj_ce_ctrl<='1';
					else
						adj_ce_ctrl<='0';
						ERR_TOO_FAST<='1';
						report "DLL IODELAY getting too fast!" severity error;
					end if;
				end if;
			elsif adj_ce='0' then
				adj_ce_ctrl<='0';
			end if;
		end if;
	end process;

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- Counter in parallel to IODELAY to monitor the number of taps
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	process(CTRL_CLK)
	begin
		if rising_edge(CTRL_CLK) then
			if RESET = '1' then
				tap_cnt<=TAP_AMOUNT;
			elsif adj_ce_ctrl ='1' then
				if inc_ctrl='1' then
					tap_cnt<=tap_cnt+1;
				else
					tap_cnt<=tap_cnt-1;
				end if;
			end if;
		end if;
	end process;

end Behavioral;

