--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  io_dll_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2008 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:
--  This module combines the adjustable oscillator with the control-circuitry
--  to form the DLL itself.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.pkg_dll_reco.all;

entity io_dll is
	Generic(
		CLOCK_BUFFER : string := "BUFR";--allowed inputs are "BUFG", "BUFR", "BUF"
		DLL_MULTIPLY: integer range 1 to 256:=2;
		TARGET_FREQUENCY: integer range 12 to 250:=125
	); 
		Port ( REFCLK : in  STD_LOGIC;
					 DLLCLK : out  STD_LOGIC;
					 LOCKED : out  STD_LOGIC;
					 RESET : in  STD_LOGIC);
end io_dll;

architecture Behavioral of io_dll is
	COMPONENT clk_adjust_v2
	Generic(
		DLL_MULTIPLY: integer range 1 to 256:=2
	); 
	PORT(
		DLL_CLK : IN std_logic;
		REF_CLK : IN std_logic;
		CTRL_CLK : IN std_logic;
		STOP_LOCK_CHECK : IN std_logic;
		RESET : IN std_logic;
		DLL_LOCKED : OUT std_logic;
		DLL_FASTER : OUT std_logic;
		DLL_SLOWER : OUT std_logic
		);
	END COMPONENT;

	COMPONENT dll_config
	Generic (
			CLOCK_BUFFER : string := "BUFR";
			TARGET_FREQUENCY: integer range 12 to 250:=125	--nominal frequency in full MHz
	);
	PORT(
		CLK_200M : IN std_logic;
		FASTER : IN std_logic;
		SLOWER : IN std_logic;
		STOP_UPDATE : IN std_logic;
		RESET : IN std_logic;
		RESET_DLY_CTRL : IN std_logic;
		CTRL_CLK : IN std_logic;
		DLL_CLK : OUT std_logic;
		DLY_CTRL_RDY : OUT std_logic
		);
	END COMPONENT;

	signal dll_faster, dll_slower: std_logic;
	signal dll_clk_int : std_logic;
begin
	clk_adjuster: clk_adjust_v2
	Generic Map(
		DLL_MULTIPLY => DLL_MULTIPLY
	)
	PORT MAP(
		DLL_CLK => dll_clk_int,
		REF_CLK => REFCLK,
		CTRL_CLK => dll_clk_int,
		STOP_LOCK_CHECK => '0',
		DLL_FASTER => dll_faster,
		DLL_SLOWER => dll_slower,
		DLL_LOCKED => LOCKED,
		RESET => RESET
	);

	configurable_dll: dll_config 
	GENERIC MAP(
		CLOCK_BUFFER => CLOCK_BUFFER,
		TARGET_FREQUENCY => TARGET_FREQUENCY
	)
	PORT MAP(
		CLK_200M => '0',
		FASTER => dll_faster,
		SLOWER => dll_slower,
		STOP_UPDATE => '0',
		DLL_CLK => dll_clk_int,
		RESET => RESET,
		RESET_DLY_CTRL => '0',
		DLY_CTRL_RDY => open,
		CTRL_CLK => dll_clk_int
	);
	DLLCLK<= dll_clk_int;
	
end Behavioral;

