--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  lock_count_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2008 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:
--  This module is run once the alignment-circuitry is switched into the
--  exact loop. It then counts for a defined number of  faster- and slower-
--  signals without FULL- or EMPTY-condition occuring.
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity lock_count is
	Generic( COUNT_LOCK : integer range 4 to 64:=16);
	Port ( FASTER : in  STD_LOGIC;
					 SLOWER : in  STD_LOGIC;
					 FULL : in  STD_LOGIC;
					 EMPTY : in  STD_LOGIC;
					 CTRL_CLK : in  STD_LOGIC;
					 CHECK_LOCK : in  STD_LOGIC;
					 LOCK : out  STD_LOGIC:='0');
end lock_count;

architecture Behavioral of lock_count is
	signal adjust_cnt: integer range 0 to COUNT_LOCK-1:=0;
begin

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- The lock-counter
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

	process(CTRL_CLK)
	begin
		if rising_edge(CTRL_CLK) then
			if CHECK_LOCK='1' then
				LOCK <= '0';
				if FULL = '1' or EMPTY ='1' then
					adjust_cnt <= 0;
				elsif (FASTER = '1') or (SLOWER = '1') then
					if adjust_cnt /= COUNT_LOCK - 1 then
						adjust_cnt <= adjust_cnt + 1;
					end if;
				end if;	
				if adjust_cnt = COUNT_LOCK - 1 then
					LOCK <= '1';
				else
					LOCK <= '0';
				end if;
			else
				adjust_cnt <= 0;
				LOCK <= '0';
			end if;
		end if;
	end process;
	
end Behavioral;

