--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  pkg_dll_reco_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2007 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

package pkg_dll_reco is

  
--User modifiable constants:

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	-- 
	-- Usage of a IDELAYCTRL-block instantiated inside the code of
	-- the oscillator if set to "true". This is advisable if other 
	-- parts of the whole design do also instantiate IDELAYCTRL-
	-- blocks.
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
  constant LOCAL_IDELAYCTRL : boolean := false;
	

	

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	--Design constants, do not change:
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	constant MAX_TAP_CNT : integer :=63;
	constant MIN_TAP_CNT : integer :=8;
	

	function start_tap_number_func (target_frequency: integer range 12 to 250) return integer;
	function dll_divide_factor_func (target_frequency: integer range 12 to 250) return integer;
	function get_input_period_func (target_frequency: integer range 12 to 250; DLL_MULTIPLY: integer range 1 to 32) return real;
	
	function round_up_func (val : REAL) RETURN integer;

	function log_bin_func (val: real) return real;
	
	function simulation_func return boolean;
	
end pkg_dll_reco;


package body pkg_dll_reco is

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- Decision if the VHDL-code is used for simulation or for
	-- synthesis
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	function simulation_func return boolean is
		variable sim: boolean;
	begin
		sim:=false;
		--pragma translate_off
		sim:=true;
		--pragma translate_on
		return sim;
	end function;
		


	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- Decision on how many taps to be used at startup
	-- Number of taps = 
	--   integer((Period(Mean(upper_limit, lower_limit))[ns] - 1 ns) / 0.16 ns)
	--
	-- example, range 200MHz .. 224MHz:
	-- * mean = 212 MHz => Period = 4.72 ns
	-- * taps = (4.72 - 1) / 0.16 = 23.25 
	-- ==> tap_amount = 23
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	function start_tap_number_func( target_frequency: integer range 12 to 250)
	return integer is
		variable tap_amount:integer range 1 to 63:=1;
	begin
			case target_frequency is
				when 225 to 250 =>
					tap_amount := 14;
				when 200 to 224 =>
					tap_amount := 17;
				when 175 to 199 =>
					tap_amount := 21;
				when 150 to 174 =>
					tap_amount := 26;
				when 135 to 149 =>
					tap_amount := 32;
				when 120 to 134 =>
					tap_amount := 37;
				when 110 to 119 =>
					tap_amount := 42;
				when  95 to 109 =>
					tap_amount := 24;
				when  85 to 94 =>
					tap_amount := 29;
				when  75 to 84 =>
					tap_amount := 33;
				when  66 to 74 =>
					tap_amount := 38;
				when  60 to 65 =>
					tap_amount := 44;
				when  54 to 59 =>
					tap_amount := 49;
				when  45 to 53 =>
					tap_amount := 29;
				when  39 to 44 =>
					tap_amount := 35;
				when  35 to 38 =>
					tap_amount := 40;
				when  31 to 34 =>
					tap_amount := 45;
				when  27 to 30 =>
					tap_amount := 52;
				when  22 to 26 =>
					tap_amount := 31;
				when  19 to 21 =>
					tap_amount := 38;
				when  16 to 18 =>
					tap_amount := 44;
				when  14 to 15 =>
					tap_amount := 52;
				when  12 to 13 =>
					tap_amount := 61;
			end case;	
		return tap_amount;
	end;


	

	function dll_divide_factor_func(target_frequency: integer range 12 to 250)
	return integer is
		variable divide_factor:integer range 0 to 3:=0;
	begin
		case target_frequency is
			when 12 to 24 =>
				divide_factor:=3;--div8
			when 25 to 54 =>
				divide_factor:=2;--div4
			when 55 to 109 =>
				divide_factor:=1;--div2
			when 110 to 250 =>
				divide_factor:=0;--div1				
		end case;
		return divide_factor;
	end;

	function get_input_period_func (TARGET_FREQUENCY: integer range 12 to 250; DLL_MULTIPLY: integer range 1 to 32) 
	return real is
		variable final_period: real;
		variable period: real;
		variable period_integer: integer;
	begin
		period := (1000.0/real(TARGET_FREQUENCY)) * real(DLL_MULTIPLY);
		period_integer :=integer(period);--avoid too many positions after decimal point
		final_period := real(period_integer);
	
		return final_period;
	end;

	function round_up_func (val : REAL) RETURN integer is
		variable val_int: integer;
		variable remainder: real;
	begin
		val_int:=integer(val);
		remainder := val - real(val_int);
		
		if remainder < 0.0 then
			return val_int;			
		else
			return val_int + 1;
		end if;
	end function;	

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- This implementation of the binary logarithm is based on the
	-- code found here:
	-- http://en.literateprograms.org/Logarithm_Function_%28Python%29
	--
	-- For hardware-implemenation it has been simplified to less
	-- accuracy.
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX	
	function log_bin_func (val: real) return real is
		constant epsilon: real := 1.0E-3;
		variable int_value: integer:=0;
		variable val_int: real;
		
		variable decfrac: real;
		variable partial : real;
		
	begin
		val_int:=val;
		
		while val_int < 1.0 loop
			int_value := int_value - 1;
			val_int := val_int * 2.0;
		end loop;
		
		while val_int >= 2.0 loop
			int_value:=int_value + 1;
			val_int := val_int / 2.0;		
		end loop;
		
		decfrac:=0.0;
		partial:=0.5;
		val_int := val_int * val_int;
		
		while partial > epsilon loop
			if val_int >= 2.0 then
				decfrac := decfrac + partial;
				val_int := val_int / 2.0;
			end if;
			partial := partial / 2.0;
			val_int := val_int * val_int;
			
		end loop;
		
		return real(int_value) + decfrac;
	end log_bin_func;

end pkg_dll_reco;
