--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  clocking_test_iodll_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2008 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:
--  This module creates the a structure of two DCMs and two BUFGMUXes that allows
--  testing of various use-models of the IO-DLL.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

library work;
use work.pkg_dll_reco.all;

entity clocking_test_iodll is
	Generic( TARGET_FREQUENCY: integer range 10 to 250:=125;
					 DLL_MULTIPLY: integer range 1 to 32:=2
	);
    Port ( REFCLK_P : in  STD_LOGIC;
           REFCLK_N : in  STD_LOGIC;
           RESET : in STD_LOGIC;
					 REFCLK_O : out  STD_LOGIC;
           REFCLK_IODLL : out  STD_LOGIC;
           DCM_CLK : out  STD_LOGIC;
           JITTERED_CLK : in  STD_LOGIC;
           DCM_LOCK : out  STD_LOGIC);
end clocking_test_iodll;

architecture Behavioral of clocking_test_iodll is
   signal CLKIN_IBUFGDS     : std_logic;
	 
	 signal JITTER_CLKFX_BUF  : std_logic;
	 signal CLKFX_SELECT      : std_logic;
	 signal JITTER_CLK_OUT    : std_logic;
	 signal JITTER_LOCK       : std_logic;

	 signal MULT_CLKFX_BUF    : std_logic;
	 signal MULT_SELECT       : std_logic;
	 signal MULT_CLK_OUT      : std_logic;
	 signal MULT_LOCK         : std_logic;

begin
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- Differential Clock-Input to FPGA
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

	REFCLK_IBUFGDS : IBUFGDS
		port map (I=>REFCLK_P,
			IB=>REFCLK_N,
			O=>CLKIN_IBUFGDS);
			
	REFCLK_O <= CLKIN_IBUFGDS;
	
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- MUXing between unaltered reference-clock and jittered 
	-- CLKFX
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	CLKFX_BUFGMUX_INST : BUFGMUX
		port map (I0=>JITTER_CLKFX_BUF,
			I1=>CLKIN_IBUFGDS,
			S=>CLKFX_SELECT,
			O=>JITTER_CLK_OUT);
   
	CLKFX_SELECT <= not JITTERED_CLK;
	REFCLK_IODLL <=JITTER_CLK_OUT;
	 
	HIGH_F_JITTER_DCM:if (TARGET_FREQUENCY >= 130) generate
		JITTER_DCM : DCM_ADV
			generic map( CLK_FEEDBACK => "NONE",
				CLKDV_DIVIDE => 2.0,
				CLKFX_DIVIDE => 32,
				CLKFX_MULTIPLY => 31,
				CLKIN_DIVIDE_BY_2 => FALSE,
				CLKIN_PERIOD => get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
				CLKOUT_PHASE_SHIFT => "NONE",
				DCM_AUTOCALIBRATION => TRUE,
				DCM_PERFORMANCE_MODE => "MAX_SPEED",
				DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
				DFS_FREQUENCY_MODE => "HIGH",
				DLL_FREQUENCY_MODE => "LOW",
				DUTY_CYCLE_CORRECTION => TRUE,
				FACTORY_JF => x"F0F0",
				PHASE_SHIFT => 0,
				STARTUP_WAIT => FALSE,
				SIM_DEVICE => "VIRTEX5")
			port map (CLKFB=>'0',
				CLKIN=>CLKIN_IBUFGDS,
				DADDR=>(others=>'0'),
				DCLK=>'0',
				DEN=>'0',
				DI=>(others=>'0'),
				DWE=>'0',
				PSCLK=>'0',
				PSEN=>'0',
				PSINCDEC=>'0',
				RST=>RESET,
				CLKDV=>open,
				CLKFX=>JITTER_CLKFX_BUF,
				CLKFX180=>open,
				CLK0=>open,
				CLK2X=>open,
				CLK2X180=>open,
				CLK90=>open,
				CLK180=>open,
				CLK270=>open,
				DO=>open,
				DRDY=>open,
				LOCKED=>JITTER_LOCK,
				PSDONE=>open);
	
	end generate;
	
	MEDIUM_F_JITTER_DCM:if (TARGET_FREQUENCY >= 40) and (TARGET_FREQUENCY < 130) generate
	 
		JITTER_DCM : DCM_ADV
			generic map( CLK_FEEDBACK => "NONE",
				CLKDV_DIVIDE => 2.0,
				CLKFX_DIVIDE => 32,
				CLKFX_MULTIPLY => 31,
				CLKIN_DIVIDE_BY_2 => FALSE,
				CLKIN_PERIOD => get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
				CLKOUT_PHASE_SHIFT => "NONE",
				DCM_AUTOCALIBRATION => TRUE,
				DCM_PERFORMANCE_MODE => "MAX_SPEED",
				DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
				DFS_FREQUENCY_MODE => "LOW",
				DLL_FREQUENCY_MODE => "LOW",
				DUTY_CYCLE_CORRECTION => TRUE,
				FACTORY_JF => x"F0F0",
				PHASE_SHIFT => 0,
				STARTUP_WAIT => FALSE,
				SIM_DEVICE => "VIRTEX5")
			port map (CLKFB=>'0',
				CLKIN=>CLKIN_IBUFGDS,
				DADDR=>(others=>'0'),
				DCLK=>'0',
				DEN=>'0',
				DI=>(others=>'0'),
				DWE=>'0',
				PSCLK=>'0',
				PSEN=>'0',
				PSINCDEC=>'0',
				RST=>RESET,
				CLKDV=>open,
				CLKFX=>JITTER_CLKFX_BUF,
				CLKFX180=>open,
				CLK0=>open,
				CLK2X=>open,
				CLK2X180=>open,
				CLK90=>open,
				CLK180=>open,
				CLK270=>open,
				DO=>open,
				DRDY=>open,
				LOCKED=>JITTER_LOCK,
				PSDONE=>open);
	
	end generate;
	
  LOW_F_JITTER_DCM:if TARGET_FREQUENCY < 40 generate
		JITTER_DCM : DCM_ADV
			generic map( CLK_FEEDBACK => "NONE",
				CLKDV_DIVIDE => 2.0,
				CLKFX_DIVIDE => 32,
				CLKFX_MULTIPLY => 31,
				CLKIN_DIVIDE_BY_2 => FALSE,
				CLKIN_PERIOD => get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
				CLKOUT_PHASE_SHIFT => "NONE",
				DCM_AUTOCALIBRATION => TRUE,
				DCM_PERFORMANCE_MODE => "MAX_RANGE",
				DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
				DFS_FREQUENCY_MODE => "LOW",
				DLL_FREQUENCY_MODE => "LOW",
				DUTY_CYCLE_CORRECTION => TRUE,
				FACTORY_JF => x"F0F0",
				PHASE_SHIFT => 0,
				STARTUP_WAIT => FALSE,
				SIM_DEVICE => "VIRTEX5")
			port map (CLKFB=>'0',
				CLKIN=>CLKIN_IBUFGDS,
				DADDR=>(others=>'0'),
				DCLK=>'0',
				DEN=>'0',
				DI=>(others=>'0'),
				DWE=>'0',
				PSCLK=>'0',
				PSEN=>'0',
				PSINCDEC=>'0',
				RST=>RESET,
				CLKDV=>open,
				CLKFX=>JITTER_CLKFX_BUF,
				CLKFX180=>open,
				CLK0=>open,
				CLK2X=>open,
				CLK2X180=>open,
				CLK90=>open,
				CLK180=>open,
				CLK270=>open,
				DO=>open,
				DRDY=>open,
				LOCKED=>JITTER_LOCK,
				PSDONE=>open);
	end generate;
				
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- MUXing between multiplied DCM-clock and jittered CLKFX from
	-- previous DCM
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

	CLKFX_BUFGMUX_CTRL_INST : BUFGMUX
		port map (I0=>MULT_CLKFX_BUF,
			I1=>JITTER_CLK_OUT,
			S=>MULT_SELECT,
			O=>MULT_CLK_OUT);
								
	MULT_SELECT <= JITTERED_CLK;
	DCM_CLK <= MULT_CLK_OUT;
	 
   
	HIGH_F_DCM:if TARGET_FREQUENCY >= 110 generate
		Jitter_clean: if DLL_MULTIPLY = 1 generate
		-- CLKFX_MULTIPLY=1 not allowed on DCM, hence use CLKFX_MULTIPLY and CLKFX_DIVIDE = 2 
			MULTIPLY_DCM_HIGH : DCM_ADV
				generic map( CLK_FEEDBACK => "NONE",
					CLKDV_DIVIDE => 2.0,
					CLKFX_DIVIDE => 2,
					CLKFX_MULTIPLY => 2,
					CLKIN_DIVIDE_BY_2 => FALSE,
					CLKIN_PERIOD => get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
					CLKOUT_PHASE_SHIFT => "NONE",
					DCM_AUTOCALIBRATION => TRUE,
					DCM_PERFORMANCE_MODE => "MAX_SPEED",
					DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
					DFS_FREQUENCY_MODE => "HIGH",
					DLL_FREQUENCY_MODE => "LOW",
					DUTY_CYCLE_CORRECTION => TRUE,
					FACTORY_JF => x"F0F0",
					PHASE_SHIFT => 0,
					STARTUP_WAIT => FALSE,
					SIM_DEVICE => "VIRTEX5")
				port map (CLKFB=>'0',
					CLKIN=>JITTER_CLK_OUT,
					DADDR=>(others=>'0'),
					DCLK=>'0',
					DEN=>'0',
					DI=>(others=>'0'),
					DWE=>'0',
					PSCLK=>'0',
					PSEN=>'0',
					PSINCDEC=>'0',
					RST=>RESET,
					CLKDV=>open,
					CLKFX=>MULT_CLKFX_BUF,
					CLKFX180=>open,
					CLK0=>open,
					CLK2X=>open,
					CLK2X180=>open,
					CLK90=>open,
					CLK180=>open,
					CLK270=>open,
					DO=>open,
					DRDY=>open,
					LOCKED=>MULT_LOCK,
					PSDONE=>open);		
		end generate;
		
		Normal_Mult: if DLL_MULTIPLY > 1 generate
			MULTIPLY_DCM_HIGH : DCM_ADV
				generic map( CLK_FEEDBACK => "NONE",
					CLKDV_DIVIDE => 2.0,
					CLKFX_DIVIDE => 1,
					CLKFX_MULTIPLY => DLL_MULTIPLY,
					CLKIN_DIVIDE_BY_2 => FALSE,
					CLKIN_PERIOD => get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
					CLKOUT_PHASE_SHIFT => "NONE",
					DCM_AUTOCALIBRATION => TRUE,
					DCM_PERFORMANCE_MODE => "MAX_SPEED",
					DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
					DFS_FREQUENCY_MODE => "HIGH",
					DLL_FREQUENCY_MODE => "LOW",
					DUTY_CYCLE_CORRECTION => TRUE,
					FACTORY_JF => x"F0F0",
					PHASE_SHIFT => 0,
					STARTUP_WAIT => FALSE,
					SIM_DEVICE => "VIRTEX5")
				port map (CLKFB=>'0',
					CLKIN=>JITTER_CLK_OUT,
					DADDR=>(others=>'0'),
					DCLK=>'0',
					DEN=>'0',
					DI=>(others=>'0'),
					DWE=>'0',
					PSCLK=>'0',
					PSEN=>'0',
					PSINCDEC=>'0',
					RST=>RESET,
					CLKDV=>open,
					CLKFX=>MULT_CLKFX_BUF,
					CLKFX180=>open,
					CLK0=>open,
					CLK2X=>open,
					CLK2X180=>open,
					CLK90=>open,
					CLK180=>open,
					CLK270=>open,
					DO=>open,
					DRDY=>open,
					LOCKED=>MULT_LOCK,
					PSDONE=>open);
		end generate;		
	end generate;
	 
	LOW_F_DCM:if TARGET_FREQUENCY < 110 generate
		Jitter_clean: if DLL_MULTIPLY = 1 generate
		-- CLKFX_MULTIPLY=1 not allowed on DCM, hence use CLKFX_MULTIPLY and CLKFX_DIVIDE = 2 
			MULTIPLY_DCM_LOW : DCM_ADV
				generic map( CLK_FEEDBACK => "NONE",
					CLKDV_DIVIDE => 2.0,
					CLKFX_DIVIDE => 2,
					CLKFX_MULTIPLY => 2,
					CLKIN_DIVIDE_BY_2 => FALSE,
					CLKIN_PERIOD => get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
					CLKOUT_PHASE_SHIFT => "NONE",
					DCM_AUTOCALIBRATION => TRUE,
					DCM_PERFORMANCE_MODE => "MAX_SPEED",
					DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
					DFS_FREQUENCY_MODE => "LOW",
					DLL_FREQUENCY_MODE => "LOW",
					DUTY_CYCLE_CORRECTION => TRUE,
					FACTORY_JF => x"F0F0",
					PHASE_SHIFT => 0,
					STARTUP_WAIT => FALSE,
					SIM_DEVICE => "VIRTEX5")
				port map (CLKFB=>'0',
					CLKIN=>JITTER_CLK_OUT,
					DADDR=>(others=>'0'),
					DCLK=>'0',
					DEN=>'0',
					DI=>(others=>'0'),
					DWE=>'0',
					PSCLK=>'0',
					PSEN=>'0',
					PSINCDEC=>'0',
					RST=>RESET,
					CLKDV=>open,
					CLKFX=>MULT_CLKFX_BUF,
					CLKFX180=>open,
					CLK0=>open,
					CLK2X=>open,
					CLK2X180=>open,
					CLK90=>open,
					CLK180=>open,
					CLK270=>open,
					DO=>open,
					DRDY=>open,
					LOCKED=>MULT_LOCK,
					PSDONE=>open);
		end generate;
		
		Normal_Mult: if DLL_MULTIPLY > 1 generate	
			MULTIPLY_DCM_LOW : DCM_ADV
				generic map( CLK_FEEDBACK => "NONE",
					CLKDV_DIVIDE => 2.0,
					CLKFX_DIVIDE => 1,
					CLKFX_MULTIPLY => DLL_MULTIPLY,
					CLKIN_DIVIDE_BY_2 => FALSE,
					CLKIN_PERIOD => get_input_period_func(TARGET_FREQUENCY=>TARGET_FREQUENCY, DLL_MULTIPLY => DLL_MULTIPLY),
					CLKOUT_PHASE_SHIFT => "NONE",
					DCM_AUTOCALIBRATION => TRUE,
					DCM_PERFORMANCE_MODE => "MAX_SPEED",
					DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS",
					DFS_FREQUENCY_MODE => "LOW",
					DLL_FREQUENCY_MODE => "LOW",
					DUTY_CYCLE_CORRECTION => TRUE,
					FACTORY_JF => x"F0F0",
					PHASE_SHIFT => 0,
					STARTUP_WAIT => FALSE,
					SIM_DEVICE => "VIRTEX5")
				port map (CLKFB=>'0',
					CLKIN=>JITTER_CLK_OUT,
					DADDR=>(others=>'0'),
					DCLK=>'0',
					DEN=>'0',
					DI=>(others=>'0'),
					DWE=>'0',
					PSCLK=>'0',
					PSEN=>'0',
					PSINCDEC=>'0',
					RST=>RESET,
					CLKDV=>open,
					CLKFX=>MULT_CLKFX_BUF,
					CLKFX180=>open,
					CLK0=>open,
					CLK2X=>open,
					CLK2X180=>open,
					CLK90=>open,
					CLK180=>open,
					CLK270=>open,
					DO=>open,
					DRDY=>open,
					LOCKED=>MULT_LOCK,
					PSDONE=>open);
		end generate;
	end generate;			

	DCM_LOCK <= JITTER_LOCK when JITTERED_CLK = '1' else
							MULT_LOCK when JITTERED_CLK = '0';

end Behavioral;

