--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  dll_osc_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2008 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:
--  This module instantiates the IDELAYCTRL-component and a primitive
--  support-system. This module is used alternatively to the one buried
--  in the design hierarchy.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity DLY_CTRL is
		Port ( CLK_200M_P : in  STD_LOGIC;
					 CLK_200M_N : in  STD_LOGIC;
					 RESET : in  STD_LOGIC;
					 RDY : out  STD_LOGIC);
end DLY_CTRL;

architecture Behavioral of DLY_CTRL is
	COMPONENT ibufds PORT(
		I: IN std_logic;
		IB: IN std_logic;
		O: OUT std_logic);
	END COMPONENT;
	COMPONENT bufg PORT(
		I: IN std_logic;
		O: OUT std_logic);
	END COMPONENT;
	COMPONENT IDELAYCTRL PORT(
		REFCLK: IN std_logic;
		RST: IN std_logic;
		RDY: OUT std_logic);
	END COMPONENT;

	signal refclk_200M_i, refclk_200M: std_logic;
	
	signal rst_cnt: std_logic_vector(4 downto 0):=(others=>'0');
	signal rst_dly_ctrl: std_logic := '1';
begin
--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
-- IDELAY_CTRL and its life-support system
--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	ref200: ibufds port map(
		I  => CLK_200M_P,
		IB => CLK_200M_N,
		O  => refclk_200M_i);
	ref200_bufg: bufg port map(
		I => refclk_200M_i,
		O => refclk_200M);
		
	dlyctrl: IDELAYCTRL port map(
		RDY    => RDY,
		REFCLK => refclk_200M,
		RST    => rst_dly_ctrl);

	process(refclk_200M)
	begin
		if rising_edge(refclk_200M) then
			if RESET = '1' then
				rst_cnt <= (others=>'0');
				rst_dly_ctrl <= '1';
			else
				if (rst_cnt = "11110") or (rst_cnt = "11111") then
					rst_dly_ctrl <= '0';
				else
					rst_cnt <= rst_cnt + '1';
					rst_dly_ctrl <='1';
				end if;
			end if;
		end if;
	end process;
end Behavioral;

