--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  io_dll_top_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2008 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all;

library XILINXCORELIB;

entity io_dll_top is
	Generic( TARGET_FREQUENCY: integer range 10 to 250:=125;
					 DLL_MULTIPLY: integer range 1 to 32:=2
	);
		Port ( REFCLK_P : in  STD_LOGIC;
					 REFCLK_N : in  STD_LOGIC;
					 CLK_200M_p: in STD_LOGIC;
					 CLK_200M_n: in STD_LOGIC;
					 IDLAYCTRLRESET : in STD_LOGIC;
					 CLKDLL_OUT_SMA_P : out  STD_LOGIC;
					 CLKDLL_OUT_SMA_N : out  STD_LOGIC;
					 CLKDLL_OUT_PIN_P : out  STD_LOGIC;
					 CLKDLL_OUT_PIN_N : out  STD_LOGIC;
					 CLKREF_OUT_P : out  STD_LOGIC;
					 CLKREF_OUT_N : out  STD_LOGIC;
					 IDLAYRDY : out STD_LOGIC;
					 ERROR : out  STD_LOGIC;
					 TOGGLE: out std_logic_vector(5 downto 0);
					 LOCK_DCM : out  STD_LOGIC;
					 LOCK_DLL : out  STD_LOGIC;
					 JITTERED_CLOCK : in STD_LOGIC;
					 RESET : in STD_LOGIC);
end io_dll_top;

architecture Behavioral of io_dll_top is
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- Basic Components
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	COMPONENT DLY_CTRL
	PORT(
		CLK_200M_P : IN std_logic;
		CLK_200M_N : IN std_logic;
		RESET : IN std_logic;
		RDY : OUT std_logic
		);
	END COMPONENT;

	COMPONENT clocking_test_iodll
	GENERIC(
		TARGET_FREQUENCY: integer range 12 to 250:=125;
		DLL_MULTIPLY: integer range 1 to 32:=2
	);
	PORT(
		REFCLK_P : IN std_logic;
		REFCLK_N : IN std_logic;
		RESET : IN std_logic;
		JITTERED_CLK : IN std_logic;          
		REFCLK_IODLL : OUT std_logic;
		REFCLK_O : OUT std_logic;
		DCM_CLK : OUT std_logic;
		DCM_LOCK : OUT std_logic
		);
	END COMPONENT;

	COMPONENT clk_out
	PORT(
		CLK : IN std_logic;
		CLK_O_P : OUT std_logic;
		CLK_O_N : OUT std_logic
		);
	END COMPONENT;
	COMPONENT reset_cleaner
	PORT(
		RESET : IN std_logic;
		RESET_DLY_CTRL : IN std_logic;
		JITTERED_CLOCK : IN std_logic;
		CLK : IN std_logic;
		RESET_CLEAN : OUT std_logic;
		RESET_DLY_CTRL_CLEAN : OUT std_logic;
		JITTERED_CLOCK_CLEAN : OUT std_logic
		);
	END COMPONENT;

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- End Basic Components
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- IO-DLL
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

	COMPONENT io_dll
	GENERIC(
		CLOCK_BUFFER : string := "BUFR";
		DLL_MULTIPLY: integer range 1 to 256:=2;
		TARGET_FREQUENCY: integer range 10 to 250:=125
		);
	PORT(
		REFCLK : IN std_logic;
		RESET : IN std_logic;
		DLLCLK : OUT std_logic;
		LOCKED : OUT std_logic
		);
	END COMPONENT;

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- End IO-DLL
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- Checkers for 0 ppm offset
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

	COMPONENT prbs_5_2v7_x7_x6
	PORT(
		CE : IN std_logic;
		RST : IN std_logic;
		CLK : IN std_logic;
		Q : OUT std_logic_vector(4 downto 0)
		);
	END COMPONENT;
	COMPONENT prbs_5_2v7_x7_x6_check
	PORT(
		CE : IN std_logic;
		RST : IN std_logic;
		CLK : IN std_logic;
		D : IN std_logic_vector(4 downto 0);
		ERROR : OUT std_logic
		);
	END COMPONENT;

	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	--
	-- End Checkers for 0 ppm offset
	--
	--XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


	COMPONENT wrrd_fifo
	PORT(
		CLK : IN std_logic;
		RESET : IN std_logic;
		WR_LOCK : IN std_logic;
		RD_LOCK : IN std_logic;
		WR_LOCK_O : OUT std_logic;
		RD_LOCK_O : OUT std_logic
		);
	END COMPONENT;

	COMPONENT x5_fifo_bram
	PORT (
		DIN: IN std_logic_VECTOR(4 downto 0);
		RD_CLK: IN std_logic;
		RD_EN: IN std_logic;
		RST: IN std_logic;
		WR_CLK: IN std_logic;
		WR_EN: IN std_logic;
		DOUT: OUT std_logic_VECTOR(4 downto 0);
		RD_DATA_COUNT : out STD_LOGIC_VECTOR ( 12 downto 0 );
		WR_DATA_COUNT : out STD_LOGIC_VECTOR ( 12 downto 0 );
		EMPTY: OUT std_logic;
		FULL: OUT std_logic;
		OVERFLOW: OUT std_logic;
		UNDERFLOW: OUT std_logic);
	END COMPONENT;

	signal rdy: std_logic;
	
	signal refclk_ibuf, refclk, dcm_clkfx: std_logic;
	signal dcm_lock: std_logic;
	signal out_clk : std_logic;

	signal dll_clk: std_logic;
	signal dll_lock: std_logic;

	signal prbs_gen, prbs_check : std_logic_vector(4 downto 0);

	signal jittered_clock_int : std_logic;
	signal error_int: std_logic;
	signal reset_int,  dly_ctrl_reset: std_logic;

	signal dcm_wr_lock, dll_rd_lock : std_logic;
	signal empty, full, overflow, underflow: std_logic;
	signal rd_data_count : STD_LOGIC_VECTOR ( 12 downto 0 );
	signal wr_data_count : STD_LOGIC_VECTOR ( 12 downto 0 );

	signal tog_cnt, tog_cnt2: std_logic_vector(27 downto 0):=(others=>'0');

begin
	IDELAY_CTRLER: DLY_CTRL PORT MAP(
		CLK_200M_P => CLK_200M_P,
		CLK_200M_N => CLK_200M_N,
		RESET =>  dly_ctrl_reset,
		RDY => rdy
	);
	IDLAYRDY<=rdy;

	CLEAN_INPUTS : reset_cleaner PORT MAP(
		RESET => RESET,
		RESET_DLY_CTRL => IDLAYCTRLRESET,
		JITTERED_CLOCK => JITTERED_CLOCK,
		RESET_CLEAN => reset_int,
		RESET_DLY_CTRL_CLEAN => dly_ctrl_reset,
		JITTERED_CLOCK_CLEAN => jittered_clock_int,
		CLK => refclk_ibuf
	);


	std_clocks: clocking_test_iodll 
	GENERIC MAP(
		TARGET_FREQUENCY => TARGET_FREQUENCY, 
		DLL_MULTIPLY => DLL_MULTIPLY
	)
	PORT MAP(
		REFCLK_P => REFCLK_P,
		REFCLK_N => REFCLK_N,
		RESET => reset_int,
		REFCLK_O => refclk_ibuf,
		REFCLK_IODLL => refclk,
		DCM_CLK => dcm_clkfx,
		JITTERED_CLK => jittered_clock_int,
		DCM_LOCK => dcm_lock
	);
	LOCK_DCM <= dcm_lock;

	iodll: io_dll
	Generic Map(
		CLOCK_BUFFER => "BUFG",
		DLL_MULTIPLY => DLL_MULTIPLY,
		TARGET_FREQUENCY => TARGET_FREQUENCY
	)
	PORT MAP(
		REFCLK => refclk,
		DLLCLK => dll_clk,
		LOCKED => dll_lock,
		RESET => reset_int
	);
	LOCK_DLL <= dll_lock;

	SMA_forward_dllclk: clk_out PORT MAP(
		CLK => dll_clk,
		CLK_O_P => CLKDLL_OUT_SMA_P,
		CLK_O_N => CLKDLL_OUT_SMA_N
	);

	PIN_forward_dllclk: clk_out PORT MAP(
		CLK => dll_clk,
		CLK_O_P => CLKDLL_OUT_PIN_P,
		CLK_O_N => CLKDLL_OUT_PIN_N
	);
	
	forward_refclk: clk_out PORT MAP(
		CLK => refclk,
		CLK_O_P => CLKREF_OUT_P,
		CLK_O_N => CLKREF_OUT_N
	);



	start_FIFO: wrrd_fifo PORT MAP(
		CLK => refclk,
		RESET => reset_int,
		WR_LOCK => dcm_lock,
		RD_LOCK => dll_lock,
		WR_LOCK_O => dcm_wr_lock,
		RD_LOCK_O => dll_rd_lock
	);



	prbs_generator: prbs_5_2v7_x7_x6 PORT MAP(
		CE => dcm_lock ,
		RST => reset_int,
		CLK => dcm_clkfx,
		Q => prbs_gen
	);

	decouple_FIFO : x5_fifo_bram
		port map (
			DIN => prbs_gen,
			WR_CLK => dcm_clkfx,
			WR_EN => dcm_wr_lock,

			RD_CLK => dll_clk,
			RD_EN => dll_rd_lock,
			DOUT => prbs_check,

			RST => reset_int,
			RD_DATA_COUNT => rd_data_count,
			WR_DATA_COUNT => wr_data_count,
			EMPTY => empty,
			FULL => full,
			OVERFLOW => overflow,
			UNDERFLOW => underflow);

	checker: prbs_5_2v7_x7_x6_check PORT MAP(
		CE => dll_rd_lock,
		RST => reset_int,
		CLK => dll_clk,
		ERROR => error_int,
		D => prbs_check
	);
	ERROR <= error_int;


	process(dll_clk)
	begin
		if rising_edge(dll_clk) then
			if reset_int='1' then
				tog_cnt <= (others=>'1');
			else
				tog_cnt<= tog_cnt +'1';
			end if;
		end if;
	end process;
	TOGGLE(0)<=tog_cnt(27);

	process(refclk)
	begin
		if rising_edge(refclk) then
			if reset_int='1' then
				tog_cnt2 <= (others=>'1');
			else
				tog_cnt2<= tog_cnt2 +'1';
			end if;
		end if;
	end process;
	TOGGLE(1)<=tog_cnt2(27);
	
	TOGGLE(5 downto 2)<=(others=>'0');

end Behavioral;

