--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  reset_cleaner_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2008 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:
--  This module is used to clean the asynchronuous inputs and avoid issues
--  inside the FPGA because of runt pulses and meta-stability.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity reset_cleaner is
		Port ( RESET : in  STD_LOGIC;
					 RESET_DLY_CTRL : in  STD_LOGIC;
					 JITTERED_CLOCK : in  STD_LOGIC;
					 RESET_CLEAN : out  STD_LOGIC:='0';
					 RESET_DLY_CTRL_CLEAN : out  STD_LOGIC:='0';
					 JITTERED_CLOCK_CLEAN : out  STD_LOGIC:='0';
					 CLK : in  STD_LOGIC);
end reset_cleaner;

architecture Behavioral of reset_cleaner is
	signal reset_int: std_logic_vector(2 downto 0);
	signal reset_dly_ctrl_int: std_logic_vector(2 downto 0);
	signal jittered_clock_int: std_logic_vector(2 downto 0);
begin

	process(CLK)
	begin
		if rising_edge(CLK) then
			reset_int(2) <= RESET;
			reset_dly_ctrl_int(2) <= RESET_DLY_CTRL;
			jittered_clock_int(2) <= JITTERED_CLOCK;
			
			reset_int(1 downto 0)<= reset_int(2 downto 1);
			reset_dly_ctrl_int(1 downto 0)<= reset_dly_ctrl_int(2 downto 1);
			jittered_clock_int(1 downto 0)<= jittered_clock_int(2 downto 1);
			
			RESET_CLEAN <= reset_int(2) and reset_int(1) and reset_int(0);
			RESET_DLY_CTRL_CLEAN <= reset_dly_ctrl_int(2) and reset_dly_ctrl_int(1) and reset_dly_ctrl_int(0);	
			JITTERED_CLOCK_CLEAN <= jittered_clock_int(2) and jittered_clock_int(1) and jittered_clock_int(0);	
			
		end if;
	end process;
			
end Behavioral;

