--
--/////////////////////////////////////////////////////////////////////////////
--
--    File Name:  wrrd_fifo_r1.0.vhd
--      Version:  1.0
--         Date:  22.02.2008
--        Model:  
--
--      Company:  Xilinx, Inc.
--  Contributor:  
--
--   Disclaimer:  XILINX IS PROVIDING THIS DESIGN, CODE, OR
--                INFORMATION "AS IS" SOLELY FOR USE IN DEVELOPING
--                PROGRAMS AND SOLUTIONS FOR XILINX DEVICES.  BY
--                PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
--                ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
--                APPLICATION OR STANDARD, XILINX IS MAKING NO
--                REPRESENTATION THAT THIS IMPLEMENTATION IS FREE
--                FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE
--                RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY
--                REQUIRE FOR YOUR IMPLEMENTATION.  XILINX
--                EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH
--                RESPECT TO THE ADEQUACY OF THE IMPLEMENTATION,
--                INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
--                REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
--                FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES
--                OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
--                PURPOSE.
--
--                (c) Copyright 2008 Xilinx, Inc.
--                All rights reserved.
--
--/////////////////////////////////////////////////////////////////////////////
-- 
-- Summary:
--	This module takes the two lock-signals from the Write- and Read-domain
--	of the checking-FIFO and creates a proper timing between them.
--	This allows that the FIFO can start at some mid-point after both clock-
--	generators are locked and avoids starting at the high- or low limit of 
--	the FIFO.
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity wrrd_fifo is
	Generic(FIFO_DEPTH : integer range 128 to 8192 := 128);
    Port ( CLK : in  STD_LOGIC;
           RESET : in  STD_LOGIC;
           WR_LOCK : in  STD_LOGIC;
           RD_LOCK : in  STD_LOGIC;
           WR_LOCK_O : out  STD_LOGIC := '0';
           RD_LOCK_O : out  STD_LOGIC := '0');
end wrrd_fifo;

architecture Behavioral of wrrd_fifo is
		
	function depth_check_func (FIFO_DEPTH : in integer range 128 to 8192) return integer is
		variable start_point_v: integer range 64 to 200 := 64;
	begin
		if FIFO_DEPTH > 200 then
			start_point_v := 100;
		else
			start_point_v := FIFO_DEPTH / 2;
		end if;	
		return start_point_v;
	end function;
	
	constant FIFO_START_POINT : integer := depth_check_func(FIFO_DEPTH);
	
	signal clk_cnt : integer range 0 to FIFO_DEPTH-1;

	signal lock_wr, lock_rd : std_logic_vector(2 downto 0) := (others=>'0');
	signal both_locked : std_logic:='0';
begin

	process (CLK)
	begin
		if rising_edge(CLK) then
			if RESET = '1' then
				lock_wr <= (others => '0'); 
				lock_rd <= (others => '0');
			else
				lock_wr(2) <= WR_LOCK;
				lock_wr(1 downto 0) <= lock_wr(2 downto 1);
			
				lock_rd(2) <= RD_LOCK;
				lock_rd(1 downto 0) <= lock_rd(2 downto 1);
			end if;
		end if;
	end process;
			



	process (CLK)
	begin
		if rising_edge(CLK) then
			if (lock_wr(0) = '1') and (lock_rd(0) = '1') then
				both_locked <= '1';
			else
				both_locked <= '0';
			end if;
		end if;
	end process;
				
	process (CLK)
	begin
		if rising_edge(CLK) then
			if both_locked = '1' then
				WR_LOCK_O <= '1';
				if clk_cnt < FIFO_START_POINT - 1 then
					clk_cnt <= clk_cnt + 1;
				end if;
			else
				WR_LOCK_O <= '0';
				clk_cnt <= 0;
			end if;
			if clk_cnt = FIFO_START_POINT -1 then
				RD_LOCK_O <='1';
			else
				RD_LOCK_O <='0';
			end if;
		end if;
	end process;
				
				
				
			

end Behavioral;

